import raypyc


LIGHTGRAY: raypyc.structures.Color
GRAY: raypyc.structures.Color
DARKGRAY: raypyc.structures.Color
YELLOW: raypyc.structures.Color
GOLD: raypyc.structures.Color
ORANGE: raypyc.structures.Color
PINK: raypyc.structures.Color
RED: raypyc.structures.Color
MAROON: raypyc.structures.Color
GREEN: raypyc.structures.Color
LIME: raypyc.structures.Color
DARKGREEN: raypyc.structures.Color
SKYBLUE: raypyc.structures.Color
BLUE: raypyc.structures.Color
DARKBLUE: raypyc.structures.Color
PURPLE: raypyc.structures.Color
VIOLET: raypyc.structures.Color
DARKPURPLE: raypyc.structures.Color
BEIGE: raypyc.structures.Color
BROWN: raypyc.structures.Color
DARKBROWN: raypyc.structures.Color
WHITE: raypyc.structures.Color
BLACK: raypyc.structures.Color
BLANK: raypyc.structures.Color
MAGENTA: raypyc.structures.Color
RAYWHITE: raypyc.structures.Color



"""**********************************************************************************************

   reasings - raylib easings library, based on Robert Penner library

   Useful easing functions for values animation

   This header uses:
       #define REASINGS_STATIC_INLINE      // Inlines all functions code, so it runs faster.
                                           // This requires lots of memory on system.
   How to use:
   The four inputs t,b,c,d are defined as follows:
   t = current time (in any unit measure, but same unit as duration)
   b = starting value to interpolate
   c = the total change in value of b that needs to occur
   d = total time it should take to complete (duration)

   Example:

   currentTime = 0
   duration = 100
   startPositionX = 0.0f
   finalPositionX = 30.0f
   currentPositionX = startPositionX

   while currentPositionX < finalPositionX:
       currentPositionX = ease_sine_in(currentTime, startPositionX, finalPositionX - startPositionX, duration)
       currentTime += 1

   A port of Robert Penner's easing equations to C (http://robertpenner.com/easing/)

   Robert Penner License
   ---------------------------------------------------------------------------------
   Open source under the BSD License.

   Copyright (c) 2001 Robert Penner. All rights reserved.

   Redistribution and use in source and binary forms, with or without modification,
   are permitted provided that the following conditions are met:

       - Redistributions of source code must retain the above copyright notice,
         this list of conditions and the following disclaimer.
       - Redistributions in binary form must reproduce the above copyright notice,
         this list of conditions and the following disclaimer in the documentation
         and/or other materials provided with the distribution.
       - Neither the name of the author nor the names of contributors may be used
         to endorse or promote products derived from this software without specific
         prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
   IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------------

   Copyright (c) 2015-2022 Ramon Santamaria (@raysan5)

   This software is provided "as-is", without any express or implied warranty. In no event
   will the authors be held liable for any damages arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose, including commercial
   applications, and to alter it and redistribute it freely, subject to the following restrictions:

     1. The origin of this software must not be misrepresented; you must not claim that you
     wrote the original software. If you use this software in a product, an acknowledgment
     in the product documentation would be appreciated but is not required.

     2. Altered source versions must be plainly marked as such, and must not be misrepresented
     as being the original software.

     3. This notice may not be removed or altered from any source distribution.

**********************************************************************************************"""

import math  # Required for: sin(), cos(), sqrt(), pow(), pi


# Linear Easing functions

def ease_linear_none(t: float, b: float, c: float, d: float) -> float:
    """Ease: Linear"""
    return c * t / d + b


def ease_linear_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Linear In"""
    return c * t / d + b


def ease_linear_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Linear Out"""
    return c * t / d + b


def ease_linear_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Linear In Out"""
    return c * t / d + b


# Sine Easing functions

def ease_sine_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Sine In"""
    return -c * math.cos(t / d * (math.pi / 2.0)) + c + b


def ease_sine_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Sine Out"""
    return c * math.sin(t / d * (math.pi / 2.0)) + b


def ease_sine_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Sine In Out"""
    return -c / 2.0 * (math.cos(math.pi * t / d) - 1.0) + b


# Circular Easing functions

def ease_circ_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Circular In"""
    t /= d
    return -c * (math.sqrt(1.0 - t * t) - 1.0) + b


def ease_circ_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Circular Out"""
    t = t / d - 1.0
    return c * math.sqrt(1.0 - t * t) + b


def ease_circ_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Circular In Out"""
    t /= d / 2.0
    if t < 1.0:
        return -c / 2.0 * (math.sqrt(1.0 - t * t) - 1.0) + b
    t -= 2.0
    return c / 2.0 * (math.sqrt(1.0 - t * t) + 1.0) + b


# Cubic Easing functions

def ease_cubic_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Cubic In"""
    t /= d
    return c * t * t * t + b


def ease_cubic_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Cubic Out"""
    t = t / d - 1.0
    return c * (t * t * t + 1.0) + b


def ease_cubic_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Cubic In Out"""
    t /= d / 2.0
    if t < 1.0:
        return c / 2.0 * t * t * t + b
    t -= 2.0
    return c / 2.0 * (t * t * t + 2.0) + b


# Quadratic Easing functions

def ease_quad_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Quadratic In"""
    t /= d
    return c * t * t + b


def ease_quad_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Quadratic Out"""
    t /= d
    return -c * t * (t - 2.0) + b


def ease_quad_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Quadratic In Out"""
    t /= d / 2
    if t < 1:
        return ((c / 2) * (t * t)) + b

    return -c / 2.0 * (((t - 1.0) * (t - 3.0)) - 1.0) + b


# Exponential Easing functions

def ease_expo_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Exponential In"""
    return b if t == 0.0 else c * math.pow(2.0, 10.0 * (t / d - 1.0)) + b


def ease_expo_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Exponential Out"""
    return b + c if t == d else c * (-math.pow(2.0, -10.0 * t / d) + 1.0) + b


def ease_expo_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Exponential In Out"""
    if t == 0.0: return b
    if t == d: return b + c
    t /= d / 2.0
    if t < 1.0: return c / 2.0 * math.pow(2.0, 10.0 * (t - 1.0)) + b

    return c / 2.0 * (-math.pow(2.0, -10.0 * (t - 1.0)) + 2.0) + b


# Back Easing functions

def ease_back_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Back In"""
    s: float = 1.70158
    t /= d
    post_fix: float = t
    return c * post_fix * t * ((s + 1.0) * t - s) + b


def ease_back_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Back Out"""
    s: float = 1.70158
    t = t / d - 1.0
    return c * (t * t * ((s + 1.0) * t + s) + 1.0) + b


def ease_back_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Back In Out"""
    s: float = 1.70158
    t /= d / 2.0

    if t < 1.0:
        s *= 1.525
        return c / 2.0 * (t * t * ((s + 1.0) * t - s)) + b

    t -= 2.0
    post_fix: float = t
    s *= 1.525
    return c / 2.0 * (post_fix * t * ((s + 1.0) * t + s) + 2.0) + b


# Bounce Easing functions

def ease_bounce_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Bounce Out"""
    t /= d
    if t < 1.0 / 2.75:
        return c * (7.5625 * t * t) + b
    elif t < 2.0 / 2.75:
        t -= 1.5 / 2.75
        post_fix: float = t
        return c * (7.5625 * post_fix * t + 0.75) + b
    elif t < 2.5 / 2.75:
        t -= 2.25 / 2.75
        post_fix: float = t
        return c * (7.5625 * post_fix * t + 0.9375) + b
    else:
        t -= 2.625 / 2.75
        post_fix: float = t
        return c * (7.5625 * post_fix * t + 0.984375) + b


def ease_bounce_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Bounce In"""
    return c - ease_bounce_out(d - t, 0.0, c, d) + b


def ease_bounce_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Bounce In Out"""
    if t < d / 2.0:
        return ease_bounce_in(t * 2.0, 0.0, c, d) * 0.5 + b
    else:
        return ease_bounce_out(t * 2.0 - d, 0.0, c, d) * 0.5 + c * 0.5 + b


# Elastic Easing functions

def ease_elastic_in(t: float, b: float, c: float, d: float) -> float:
    """Ease: Elastic In"""
    if t == 0.0: return b
    t /= d
    if t == 1.0: return b + c

    p: float = d * 0.3
    a: float = c
    s: float = p / 4.0
    t -= 1.0
    post_fix: float = a * math.pow(2.0, 10.0 * t)

    return -(post_fix * math.sin((t * d - s) * (2.0 * math.pi) / p)) + b


def ease_elastic_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Elastic Out"""
    if t == 0.0: return b
    t /= d
    if t == 1.0: return b + c

    p: float = d * 0.3
    a: float = c
    s: float = p / 4.0

    return a * math.pow(2.0, -10.0 * t) * math.sin((t * d - s) * (2.0 * math.pi) / p) + c + b


def ease_elastic_in_out(t: float, b: float, c: float, d: float) -> float:
    """Ease: Elastic In Out"""
    if t == 0.0: return b
    t /= d / 2.0
    if t == 2.0: return b + c

    p: float = d * (0.3 * 1.5)
    a: float = c
    s: float = p / 4.0

    if t < 1.0:
        t -= 1.0
        post_fix: float = a * math.pow(2.0, 10.0 * t)
        return -0.5 * (post_fix * math.sin((t * d - s) * (2.0 * math.pi) / p)) + b

    t -= 1.0
    post_fix: float = a * math.pow(2.0, -10.0 * t)

    return post_fix * math.sin((t * d - s) * (2.0 * math.pi) / p) * 0.5 + c + b



import raypyc


LIGHTGRAY: raypyc.structures.Color = raypyc.structures.Color(200, 200, 200, 255)  # Light Gray
GRAY: raypyc.structures.Color = raypyc.structures.Color(130, 130, 130, 255)  # Gray
DARKGRAY: raypyc.structures.Color = raypyc.structures.Color(80, 80, 80, 255)  # Dark Gray
YELLOW: raypyc.structures.Color = raypyc.structures.Color(253, 249, 0, 255)  # Yellow
GOLD: raypyc.structures.Color = raypyc.structures.Color(255, 203, 0, 255)  # Gold
ORANGE: raypyc.structures.Color = raypyc.structures.Color(255, 161, 0, 255)  # Orange
PINK: raypyc.structures.Color = raypyc.structures.Color(255, 109, 194, 255)  # Pink
RED: raypyc.structures.Color = raypyc.structures.Color(230, 41, 55, 255)  # Red
MAROON: raypyc.structures.Color = raypyc.structures.Color(190, 33, 55, 255)  # Maroon
GREEN: raypyc.structures.Color = raypyc.structures.Color(0, 228, 48, 255)  # Green
LIME: raypyc.structures.Color = raypyc.structures.Color(0, 158, 47, 255)  # Lime
DARKGREEN: raypyc.structures.Color = raypyc.structures.Color(0, 117, 44, 255)  # Dark Green
SKYBLUE: raypyc.structures.Color = raypyc.structures.Color(102, 191, 255, 255)  # Sky Blue
BLUE: raypyc.structures.Color = raypyc.structures.Color(0, 121, 241, 255)  # Blue
DARKBLUE: raypyc.structures.Color = raypyc.structures.Color(0, 82, 172, 255)  # Dark Blue
PURPLE: raypyc.structures.Color = raypyc.structures.Color(200, 122, 255, 255)  # Purple
VIOLET: raypyc.structures.Color = raypyc.structures.Color(135, 60, 190, 255)  # Violet
DARKPURPLE: raypyc.structures.Color = raypyc.structures.Color(112, 31, 126, 255)  # Dark Purple
BEIGE: raypyc.structures.Color = raypyc.structures.Color(211, 176, 131, 255)  # Beige
BROWN: raypyc.structures.Color = raypyc.structures.Color(127, 106, 79, 255)  # Brown
DARKBROWN: raypyc.structures.Color = raypyc.structures.Color(76, 63, 47, 255)  # Dark Brown
WHITE: raypyc.structures.Color = raypyc.structures.Color(255, 255, 255, 255)  # White
BLACK: raypyc.structures.Color = raypyc.structures.Color(0, 0, 0, 255)  # Black
BLANK: raypyc.structures.Color = raypyc.structures.Color(0, 0, 0, 0)  # Blank (Transparent)
MAGENTA: raypyc.structures.Color = raypyc.structures.Color(255, 0, 255, 255)  # Magenta
RAYWHITE: raypyc.structures.Color = raypyc.structures.Color(245, 245, 245, 255)  # My own White (raylib logo)



import ctypes
from raypyc.defines import *


class rAudioBuffer(ctypes.Structure):
	"""dummy structure"""
	_fields_ = [
		("data", ctypes.c_byte * 24)
	]


class rAudioProcessor(ctypes.Structure):
	"""dummy structure"""
	_fields_ = [
		("data", ctypes.c_byte * 24)
	]


class rlVertexBuffer(ctypes.Structure):
	"""Dynamic vertex buffers (position + texcoords + colors + indices arrays)"""
	_fields_ = [
		('elementCount', ctypes.c_int),  # Number of elements in the buffer (QUADS)
		('vertices', ctypes.POINTER(ctypes.c_float)),  # Vertex position (XYZ - 3 components per vertex) (shader-location = 0)
		('texcoords', ctypes.POINTER(ctypes.c_float)),  # Vertex texture coordinates (UV - 2 components per vertex) (shader-location = 1)
		('colors', ctypes.POINTER(ctypes.c_ubyte)),  # Vertex colors (RGBA - 4 components per vertex) (shader-location = 3)
		('indices', ctypes.POINTER(ctypes.c_uint)),  # Vertex indices (in case vertex data comes indexed) (6 indices per quad)
		('vaoId', ctypes.c_uint),  # OpenGL Vertex Array Object id
		('vboId', ctypes.c_uint * 4)  # OpenGL Vertex Buffer Objects id (4 types of vertex data)
	]

	@property
	def elementCount(self) -> ctypes.c_int:
		return self.elementCount

	@elementCount.setter
	def elementCount(self, i: ctypes.c_int) -> None:
		self.elementCount = i

	@property
	def vertices(self) -> ctypes.POINTER(ctypes.c_float):
		return self.vertices

	@vertices.setter
	def vertices(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.vertices = i

	@property
	def texcoords(self) -> ctypes.POINTER(ctypes.c_float):
		return self.texcoords

	@texcoords.setter
	def texcoords(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.texcoords = i

	@property
	def colors(self) -> ctypes.POINTER(ctypes.c_ubyte):
		return self.colors

	@colors.setter
	def colors(self, i: ctypes.POINTER(ctypes.c_ubyte)) -> None:
		self.colors = i

	@property
	def indices(self) -> ctypes.POINTER(ctypes.c_uint):
		return self.indices

	@indices.setter
	def indices(self, i: ctypes.POINTER(ctypes.c_uint)) -> None:
		self.indices = i

	@property
	def vaoId(self) -> ctypes.c_uint:
		return self.vaoId

	@vaoId.setter
	def vaoId(self, i: ctypes.c_uint) -> None:
		self.vaoId = i

	@property
	def vboId(self) -> ctypes.c_uint * 4:
		return self.vboId

	@vboId.setter
	def vboId(self, i: ctypes.c_uint * 4) -> None:
		self.vboId = i


class rlDrawCall(ctypes.Structure):
	"""of those state-change happens (this is done in core module)"""
	_fields_ = [
		('mode', ctypes.c_int),  # Drawing mode: LINES, TRIANGLES, QUADS
		('vertexCount', ctypes.c_int),  # Number of vertex of the draw
		('vertexAlignment', ctypes.c_int),  # Number of vertex required for index alignment (LINES, TRIANGLES)
		('textureId', ctypes.c_uint)  # Texture id to be used on the draw -> Use to create new draw call if changes
	]

	@property
	def mode(self) -> ctypes.c_int:
		return self.mode

	@mode.setter
	def mode(self, i: ctypes.c_int) -> None:
		self.mode = i

	@property
	def vertexCount(self) -> ctypes.c_int:
		return self.vertexCount

	@vertexCount.setter
	def vertexCount(self, i: ctypes.c_int) -> None:
		self.vertexCount = i

	@property
	def vertexAlignment(self) -> ctypes.c_int:
		return self.vertexAlignment

	@vertexAlignment.setter
	def vertexAlignment(self, i: ctypes.c_int) -> None:
		self.vertexAlignment = i

	@property
	def textureId(self) -> ctypes.c_uint:
		return self.textureId

	@textureId.setter
	def textureId(self, i: ctypes.c_uint) -> None:
		self.textureId = i


class rlRenderBatch(ctypes.Structure):
	"""rlRenderBatch type"""
	_fields_ = [
		('bufferCount', ctypes.c_int),  # Number of vertex buffers (multi-buffering support)
		('currentBuffer', ctypes.c_int),  # Current buffer tracking in case of multi-buffering
		('vertexBuffer', ctypes.POINTER(rlVertexBuffer)),  # Dynamic buffer(s) for vertex data
		('draws', ctypes.POINTER(rlDrawCall)),  # Draw calls array, depends on textureId
		('drawCounter', ctypes.c_int),  # Draw calls counter
		('currentDepth', ctypes.c_float)  # Current depth value for next draw
	]

	@property
	def bufferCount(self) -> ctypes.c_int:
		return self.bufferCount

	@bufferCount.setter
	def bufferCount(self, i: ctypes.c_int) -> None:
		self.bufferCount = i

	@property
	def currentBuffer(self) -> ctypes.c_int:
		return self.currentBuffer

	@currentBuffer.setter
	def currentBuffer(self, i: ctypes.c_int) -> None:
		self.currentBuffer = i

	@property
	def vertexBuffer(self) -> ctypes.POINTER(rlVertexBuffer):
		return self.vertexBuffer

	@vertexBuffer.setter
	def vertexBuffer(self, i: ctypes.POINTER(rlVertexBuffer)) -> None:
		self.vertexBuffer = i

	@property
	def draws(self) -> ctypes.POINTER(rlDrawCall):
		return self.draws

	@draws.setter
	def draws(self, i: ctypes.POINTER(rlDrawCall)) -> None:
		self.draws = i

	@property
	def drawCounter(self) -> ctypes.c_int:
		return self.drawCounter

	@drawCounter.setter
	def drawCounter(self, i: ctypes.c_int) -> None:
		self.drawCounter = i

	@property
	def currentDepth(self) -> ctypes.c_float:
		return self.currentDepth

	@currentDepth.setter
	def currentDepth(self, i: ctypes.c_float) -> None:
		self.currentDepth = i


class Matrix(ctypes.Structure):
	"""Matrix, 4x4 components, column major, OpenGL style, right handed"""
	_fields_ = [
		('m0', ctypes.c_float),  # Matrix first row (4 components)
		('m4', ctypes.c_float),  # Matrix first row (4 components)
		('m8', ctypes.c_float),  # Matrix first row (4 components)
		('m12', ctypes.c_float),  # Matrix first row (4 components)
		('m1', ctypes.c_float),  # Matrix second row (4 components)
		('m5', ctypes.c_float),  # Matrix second row (4 components)
		('m9', ctypes.c_float),  # Matrix second row (4 components)
		('m13', ctypes.c_float),  # Matrix second row (4 components)
		('m2', ctypes.c_float),  # Matrix third row (4 components)
		('m6', ctypes.c_float),  # Matrix third row (4 components)
		('m10', ctypes.c_float),  # Matrix third row (4 components)
		('m14', ctypes.c_float),  # Matrix third row (4 components)
		('m3', ctypes.c_float),  # Matrix fourth row (4 components)
		('m7', ctypes.c_float),  # Matrix fourth row (4 components)
		('m11', ctypes.c_float),  # Matrix fourth row (4 components)
		('m15', ctypes.c_float)  # Matrix fourth row (4 components)
	]

	@property
	def m0(self) -> ctypes.c_float:
		return self.m0

	@m0.setter
	def m0(self, i: ctypes.c_float) -> None:
		self.m0 = i

	@property
	def m4(self) -> ctypes.c_float:
		return self.m4

	@m4.setter
	def m4(self, i: ctypes.c_float) -> None:
		self.m4 = i

	@property
	def m8(self) -> ctypes.c_float:
		return self.m8

	@m8.setter
	def m8(self, i: ctypes.c_float) -> None:
		self.m8 = i

	@property
	def m12(self) -> ctypes.c_float:
		return self.m12

	@m12.setter
	def m12(self, i: ctypes.c_float) -> None:
		self.m12 = i

	@property
	def m1(self) -> ctypes.c_float:
		return self.m1

	@m1.setter
	def m1(self, i: ctypes.c_float) -> None:
		self.m1 = i

	@property
	def m5(self) -> ctypes.c_float:
		return self.m5

	@m5.setter
	def m5(self, i: ctypes.c_float) -> None:
		self.m5 = i

	@property
	def m9(self) -> ctypes.c_float:
		return self.m9

	@m9.setter
	def m9(self, i: ctypes.c_float) -> None:
		self.m9 = i

	@property
	def m13(self) -> ctypes.c_float:
		return self.m13

	@m13.setter
	def m13(self, i: ctypes.c_float) -> None:
		self.m13 = i

	@property
	def m2(self) -> ctypes.c_float:
		return self.m2

	@m2.setter
	def m2(self, i: ctypes.c_float) -> None:
		self.m2 = i

	@property
	def m6(self) -> ctypes.c_float:
		return self.m6

	@m6.setter
	def m6(self, i: ctypes.c_float) -> None:
		self.m6 = i

	@property
	def m10(self) -> ctypes.c_float:
		return self.m10

	@m10.setter
	def m10(self, i: ctypes.c_float) -> None:
		self.m10 = i

	@property
	def m14(self) -> ctypes.c_float:
		return self.m14

	@m14.setter
	def m14(self, i: ctypes.c_float) -> None:
		self.m14 = i

	@property
	def m3(self) -> ctypes.c_float:
		return self.m3

	@m3.setter
	def m3(self, i: ctypes.c_float) -> None:
		self.m3 = i

	@property
	def m7(self) -> ctypes.c_float:
		return self.m7

	@m7.setter
	def m7(self, i: ctypes.c_float) -> None:
		self.m7 = i

	@property
	def m11(self) -> ctypes.c_float:
		return self.m11

	@m11.setter
	def m11(self, i: ctypes.c_float) -> None:
		self.m11 = i

	@property
	def m15(self) -> ctypes.c_float:
		return self.m15

	@m15.setter
	def m15(self, i: ctypes.c_float) -> None:
		self.m15 = i


class rlglData(ctypes.Structure):
	""""""
	_fields_ = [
		('currentBatch', ctypes.POINTER(rlRenderBatch)),  # Current render batch
		('defaultBatch', rlRenderBatch),  # Default internal render batch
		('vertexCounter', ctypes.c_int),  # Current active render batch vertex counter (generic, used for all batches)
		('texcoordx', ctypes.c_float),  # Current active texture coordinate (added on glVertex*())
		('texcoordy', ctypes.c_float),  # Current active texture coordinate (added on glVertex*())
		('normalx', ctypes.c_float),  # Current active normal (added on glVertex*())
		('normaly', ctypes.c_float),  # Current active normal (added on glVertex*())
		('normalz', ctypes.c_float),  # Current active normal (added on glVertex*())
		('colorr', ctypes.c_ubyte),  # Current active color (added on glVertex*())
		('colorg', ctypes.c_ubyte),  # Current active color (added on glVertex*())
		('colorb', ctypes.c_ubyte),  # Current active color (added on glVertex*())
		('colora', ctypes.c_ubyte),  # Current active color (added on glVertex*())
		('currentMatrixMode', ctypes.c_int),  # Current matrix mode
		('currentMatrix', ctypes.POINTER(Matrix)),  # Current matrix pointer
		('modelview', Matrix),  # Default modelview matrix
		('projection', Matrix),  # Default projection matrix
		('transform', Matrix),  # Transform matrix to be used with rlTranslate, rlRotate, rlScale
		('transformRequired', ctypes.c_bool),  # Require transform matrix application to current draw-call vertex (if required)
		('stack', Matrix * RL_MAX_MATRIX_STACK_SIZE),  # Matrix stack for push/pop
		('stackCounter', ctypes.c_int),  # Matrix stack counter
		('defaultTextureId', ctypes.c_uint),  # Default texture used on shapes/poly drawing (required by shader)
		('activeTextureId', ctypes.c_uint * RL_DEFAULT_BATCH_MAX_TEXTURE_UNITS),  # Active texture ids to be enabled on batch drawing (0 active by default)
		('defaultVShaderId', ctypes.c_uint),  # Default vertex shader id (used by default shader program)
		('defaultFShaderId', ctypes.c_uint),  # Default fragment shader id (used by default shader program)
		('defaultShaderId', ctypes.c_uint),  # Default shader program id, supports vertex color and diffuse texture
		('defaultShaderLocs', ctypes.POINTER(ctypes.c_int)),  # Default shader locations pointer to be used on rendering
		('currentShaderId', ctypes.c_uint),  # Current shader id to be used on rendering (by default, defaultShaderId)
		('currentShaderLocs', ctypes.POINTER(ctypes.c_int)),  # Current shader locations pointer to be used on rendering (by default, defaultShaderLocs)
		('stereoRender', ctypes.c_bool),  # Stereo rendering flag
		('projectionStereo', Matrix * 2),  # VR stereo rendering eyes projection matrices
		('viewOffsetStereo', Matrix * 2),  # VR stereo rendering eyes view offset matrices
		('currentBlendMode', ctypes.c_int),  # Blending mode active
		('glBlendSrcFactor', ctypes.c_int),  # Blending source factor
		('glBlendDstFactor', ctypes.c_int),  # Blending destination factor
		('glBlendEquation', ctypes.c_int),  # Blending equation
		('framebufferWidth', ctypes.c_int),  # Current framebuffer width
		('framebufferHeight', ctypes.c_int)  # Current framebuffer height
	]

	@property
	def currentBatch(self) -> ctypes.POINTER(rlRenderBatch):
		return self.currentBatch

	@currentBatch.setter
	def currentBatch(self, i: ctypes.POINTER(rlRenderBatch)) -> None:
		self.currentBatch = i

	@property
	def defaultBatch(self) -> rlRenderBatch:
		return self.defaultBatch

	@defaultBatch.setter
	def defaultBatch(self, i: rlRenderBatch) -> None:
		self.defaultBatch = i

	@property
	def vertexCounter(self) -> ctypes.c_int:
		return self.vertexCounter

	@vertexCounter.setter
	def vertexCounter(self, i: ctypes.c_int) -> None:
		self.vertexCounter = i

	@property
	def texcoordx(self) -> ctypes.c_float:
		return self.texcoordx

	@texcoordx.setter
	def texcoordx(self, i: ctypes.c_float) -> None:
		self.texcoordx = i

	@property
	def texcoordy(self) -> ctypes.c_float:
		return self.texcoordy

	@texcoordy.setter
	def texcoordy(self, i: ctypes.c_float) -> None:
		self.texcoordy = i

	@property
	def normalx(self) -> ctypes.c_float:
		return self.normalx

	@normalx.setter
	def normalx(self, i: ctypes.c_float) -> None:
		self.normalx = i

	@property
	def normaly(self) -> ctypes.c_float:
		return self.normaly

	@normaly.setter
	def normaly(self, i: ctypes.c_float) -> None:
		self.normaly = i

	@property
	def normalz(self) -> ctypes.c_float:
		return self.normalz

	@normalz.setter
	def normalz(self, i: ctypes.c_float) -> None:
		self.normalz = i

	@property
	def colorr(self) -> ctypes.c_ubyte:
		return self.colorr

	@colorr.setter
	def colorr(self, i: ctypes.c_ubyte) -> None:
		self.colorr = i

	@property
	def colorg(self) -> ctypes.c_ubyte:
		return self.colorg

	@colorg.setter
	def colorg(self, i: ctypes.c_ubyte) -> None:
		self.colorg = i

	@property
	def colorb(self) -> ctypes.c_ubyte:
		return self.colorb

	@colorb.setter
	def colorb(self, i: ctypes.c_ubyte) -> None:
		self.colorb = i

	@property
	def colora(self) -> ctypes.c_ubyte:
		return self.colora

	@colora.setter
	def colora(self, i: ctypes.c_ubyte) -> None:
		self.colora = i

	@property
	def currentMatrixMode(self) -> ctypes.c_int:
		return self.currentMatrixMode

	@currentMatrixMode.setter
	def currentMatrixMode(self, i: ctypes.c_int) -> None:
		self.currentMatrixMode = i

	@property
	def currentMatrix(self) -> ctypes.POINTER(Matrix):
		return self.currentMatrix

	@currentMatrix.setter
	def currentMatrix(self, i: ctypes.POINTER(Matrix)) -> None:
		self.currentMatrix = i

	@property
	def modelview(self) -> Matrix:
		return self.modelview

	@modelview.setter
	def modelview(self, i: Matrix) -> None:
		self.modelview = i

	@property
	def projection(self) -> Matrix:
		return self.projection

	@projection.setter
	def projection(self, i: Matrix) -> None:
		self.projection = i

	@property
	def transform(self) -> Matrix:
		return self.transform

	@transform.setter
	def transform(self, i: Matrix) -> None:
		self.transform = i

	@property
	def transformRequired(self) -> ctypes.c_bool:
		return self.transformRequired

	@transformRequired.setter
	def transformRequired(self, i: ctypes.c_bool) -> None:
		self.transformRequired = i

	@property
	def stack(self) -> Matrix * RL_MAX_MATRIX_STACK_SIZE:
		return self.stack

	@stack.setter
	def stack(self, i: Matrix * RL_MAX_MATRIX_STACK_SIZE) -> None:
		self.stack = i

	@property
	def stackCounter(self) -> ctypes.c_int:
		return self.stackCounter

	@stackCounter.setter
	def stackCounter(self, i: ctypes.c_int) -> None:
		self.stackCounter = i

	@property
	def defaultTextureId(self) -> ctypes.c_uint:
		return self.defaultTextureId

	@defaultTextureId.setter
	def defaultTextureId(self, i: ctypes.c_uint) -> None:
		self.defaultTextureId = i

	@property
	def activeTextureId(self) -> ctypes.c_uint * RL_DEFAULT_BATCH_MAX_TEXTURE_UNITS:
		return self.activeTextureId

	@activeTextureId.setter
	def activeTextureId(self, i: ctypes.c_uint * RL_DEFAULT_BATCH_MAX_TEXTURE_UNITS) -> None:
		self.activeTextureId = i

	@property
	def defaultVShaderId(self) -> ctypes.c_uint:
		return self.defaultVShaderId

	@defaultVShaderId.setter
	def defaultVShaderId(self, i: ctypes.c_uint) -> None:
		self.defaultVShaderId = i

	@property
	def defaultFShaderId(self) -> ctypes.c_uint:
		return self.defaultFShaderId

	@defaultFShaderId.setter
	def defaultFShaderId(self, i: ctypes.c_uint) -> None:
		self.defaultFShaderId = i

	@property
	def defaultShaderId(self) -> ctypes.c_uint:
		return self.defaultShaderId

	@defaultShaderId.setter
	def defaultShaderId(self, i: ctypes.c_uint) -> None:
		self.defaultShaderId = i

	@property
	def defaultShaderLocs(self) -> ctypes.POINTER(ctypes.c_int):
		return self.defaultShaderLocs

	@defaultShaderLocs.setter
	def defaultShaderLocs(self, i: ctypes.POINTER(ctypes.c_int)) -> None:
		self.defaultShaderLocs = i

	@property
	def currentShaderId(self) -> ctypes.c_uint:
		return self.currentShaderId

	@currentShaderId.setter
	def currentShaderId(self, i: ctypes.c_uint) -> None:
		self.currentShaderId = i

	@property
	def currentShaderLocs(self) -> ctypes.POINTER(ctypes.c_int):
		return self.currentShaderLocs

	@currentShaderLocs.setter
	def currentShaderLocs(self, i: ctypes.POINTER(ctypes.c_int)) -> None:
		self.currentShaderLocs = i

	@property
	def stereoRender(self) -> ctypes.c_bool:
		return self.stereoRender

	@stereoRender.setter
	def stereoRender(self, i: ctypes.c_bool) -> None:
		self.stereoRender = i

	@property
	def projectionStereo(self) -> Matrix * 2:
		return self.projectionStereo

	@projectionStereo.setter
	def projectionStereo(self, i: Matrix * 2) -> None:
		self.projectionStereo = i

	@property
	def viewOffsetStereo(self) -> Matrix * 2:
		return self.viewOffsetStereo

	@viewOffsetStereo.setter
	def viewOffsetStereo(self, i: Matrix * 2) -> None:
		self.viewOffsetStereo = i

	@property
	def currentBlendMode(self) -> ctypes.c_int:
		return self.currentBlendMode

	@currentBlendMode.setter
	def currentBlendMode(self, i: ctypes.c_int) -> None:
		self.currentBlendMode = i

	@property
	def glBlendSrcFactor(self) -> ctypes.c_int:
		return self.glBlendSrcFactor

	@glBlendSrcFactor.setter
	def glBlendSrcFactor(self, i: ctypes.c_int) -> None:
		self.glBlendSrcFactor = i

	@property
	def glBlendDstFactor(self) -> ctypes.c_int:
		return self.glBlendDstFactor

	@glBlendDstFactor.setter
	def glBlendDstFactor(self, i: ctypes.c_int) -> None:
		self.glBlendDstFactor = i

	@property
	def glBlendEquation(self) -> ctypes.c_int:
		return self.glBlendEquation

	@glBlendEquation.setter
	def glBlendEquation(self, i: ctypes.c_int) -> None:
		self.glBlendEquation = i

	@property
	def framebufferWidth(self) -> ctypes.c_int:
		return self.framebufferWidth

	@framebufferWidth.setter
	def framebufferWidth(self, i: ctypes.c_int) -> None:
		self.framebufferWidth = i

	@property
	def framebufferHeight(self) -> ctypes.c_int:
		return self.framebufferHeight

	@framebufferHeight.setter
	def framebufferHeight(self, i: ctypes.c_int) -> None:
		self.framebufferHeight = i


class Vector2(ctypes.Structure):
	"""Vector2, 2 components"""
	_fields_ = [
		('x', ctypes.c_float),  # Vector x component
		('y', ctypes.c_float)  # Vector y component
	]

	@property
	def x(self) -> ctypes.c_float:
		return self.x

	@x.setter
	def x(self, i: ctypes.c_float) -> None:
		self.x = i

	@property
	def y(self) -> ctypes.c_float:
		return self.y

	@y.setter
	def y(self, i: ctypes.c_float) -> None:
		self.y = i


class Vector3(ctypes.Structure):
	"""Vector3, 3 components"""
	_fields_ = [
		('x', ctypes.c_float),  # Vector x component
		('y', ctypes.c_float),  # Vector y component
		('z', ctypes.c_float)  # Vector z component
	]

	@property
	def x(self) -> ctypes.c_float:
		return self.x

	@x.setter
	def x(self, i: ctypes.c_float) -> None:
		self.x = i

	@property
	def y(self) -> ctypes.c_float:
		return self.y

	@y.setter
	def y(self, i: ctypes.c_float) -> None:
		self.y = i

	@property
	def z(self) -> ctypes.c_float:
		return self.z

	@z.setter
	def z(self, i: ctypes.c_float) -> None:
		self.z = i


class Vector4(ctypes.Structure):
	"""Vector4, 4 components"""
	_fields_ = [
		('x', ctypes.c_float),  # Vector x component
		('y', ctypes.c_float),  # Vector y component
		('z', ctypes.c_float),  # Vector z component
		('w', ctypes.c_float)  # Vector w component
	]

	@property
	def x(self) -> ctypes.c_float:
		return self.x

	@x.setter
	def x(self, i: ctypes.c_float) -> None:
		self.x = i

	@property
	def y(self) -> ctypes.c_float:
		return self.y

	@y.setter
	def y(self, i: ctypes.c_float) -> None:
		self.y = i

	@property
	def z(self) -> ctypes.c_float:
		return self.z

	@z.setter
	def z(self, i: ctypes.c_float) -> None:
		self.z = i

	@property
	def w(self) -> ctypes.c_float:
		return self.w

	@w.setter
	def w(self, i: ctypes.c_float) -> None:
		self.w = i


class Quaternion(ctypes.Structure):
	"""Quaternion, 4 components (Vector4 alias)"""
	_fields_ = [
		('x', ctypes.c_float),  # Vector x component
		('y', ctypes.c_float),  # Vector y component
		('z', ctypes.c_float),  # Vector z component
		('w', ctypes.c_float)  # Vector w component
	]

	@property
	def x(self) -> ctypes.c_float:
		return self.x

	@x.setter
	def x(self, i: ctypes.c_float) -> None:
		self.x = i

	@property
	def y(self) -> ctypes.c_float:
		return self.y

	@y.setter
	def y(self, i: ctypes.c_float) -> None:
		self.y = i

	@property
	def z(self) -> ctypes.c_float:
		return self.z

	@z.setter
	def z(self, i: ctypes.c_float) -> None:
		self.z = i

	@property
	def w(self) -> ctypes.c_float:
		return self.w

	@w.setter
	def w(self, i: ctypes.c_float) -> None:
		self.w = i


class Color(ctypes.Structure):
	"""Color, 4 components, R8G8B8A8 (32bit)"""
	_fields_ = [
		('r', ctypes.c_ubyte),  # Color red value
		('g', ctypes.c_ubyte),  # Color green value
		('b', ctypes.c_ubyte),  # Color blue value
		('a', ctypes.c_ubyte)  # Color alpha value
	]

	@property
	def r(self) -> ctypes.c_ubyte:
		return self.r

	@r.setter
	def r(self, i: ctypes.c_ubyte) -> None:
		self.r = i

	@property
	def g(self) -> ctypes.c_ubyte:
		return self.g

	@g.setter
	def g(self, i: ctypes.c_ubyte) -> None:
		self.g = i

	@property
	def b(self) -> ctypes.c_ubyte:
		return self.b

	@b.setter
	def b(self, i: ctypes.c_ubyte) -> None:
		self.b = i

	@property
	def a(self) -> ctypes.c_ubyte:
		return self.a

	@a.setter
	def a(self, i: ctypes.c_ubyte) -> None:
		self.a = i


class Rectangle(ctypes.Structure):
	"""Rectangle, 4 components"""
	_fields_ = [
		('x', ctypes.c_float),  # Rectangle top-left corner position x
		('y', ctypes.c_float),  # Rectangle top-left corner position y
		('width', ctypes.c_float),  # Rectangle width
		('height', ctypes.c_float)  # Rectangle height
	]

	@property
	def x(self) -> ctypes.c_float:
		return self.x

	@x.setter
	def x(self, i: ctypes.c_float) -> None:
		self.x = i

	@property
	def y(self) -> ctypes.c_float:
		return self.y

	@y.setter
	def y(self, i: ctypes.c_float) -> None:
		self.y = i

	@property
	def width(self) -> ctypes.c_float:
		return self.width

	@width.setter
	def width(self, i: ctypes.c_float) -> None:
		self.width = i

	@property
	def height(self) -> ctypes.c_float:
		return self.height

	@height.setter
	def height(self, i: ctypes.c_float) -> None:
		self.height = i


class Image(ctypes.Structure):
	"""Image, pixel data stored in CPU memory (RAM)"""
	_fields_ = [
		('data', ctypes.c_void_p),  # Image raw data
		('width', ctypes.c_int),  # Image base width
		('height', ctypes.c_int),  # Image base height
		('mipmaps', ctypes.c_int),  # Mipmap levels, 1 by default
		('format', ctypes.c_int)  # Data format (PixelFormat type)
	]

	@property
	def data(self) -> ctypes.c_void_p:
		return self.data

	@data.setter
	def data(self, i: ctypes.c_void_p) -> None:
		self.data = i

	@property
	def width(self) -> ctypes.c_int:
		return self.width

	@width.setter
	def width(self, i: ctypes.c_int) -> None:
		self.width = i

	@property
	def height(self) -> ctypes.c_int:
		return self.height

	@height.setter
	def height(self, i: ctypes.c_int) -> None:
		self.height = i

	@property
	def mipmaps(self) -> ctypes.c_int:
		return self.mipmaps

	@mipmaps.setter
	def mipmaps(self, i: ctypes.c_int) -> None:
		self.mipmaps = i

	@property
	def format(self) -> ctypes.c_int:
		return self.format

	@format.setter
	def format(self, i: ctypes.c_int) -> None:
		self.format = i


class Texture(ctypes.Structure):
	"""Texture, tex data stored in GPU memory (VRAM)"""
	_fields_ = [
		('id', ctypes.c_uint),  # OpenGL texture id
		('width', ctypes.c_int),  # Texture base width
		('height', ctypes.c_int),  # Texture base height
		('mipmaps', ctypes.c_int),  # Mipmap levels, 1 by default
		('format', ctypes.c_int)  # Data format (PixelFormat type)
	]

	@property
	def id(self) -> ctypes.c_uint:
		return self.id

	@id.setter
	def id(self, i: ctypes.c_uint) -> None:
		self.id = i

	@property
	def width(self) -> ctypes.c_int:
		return self.width

	@width.setter
	def width(self, i: ctypes.c_int) -> None:
		self.width = i

	@property
	def height(self) -> ctypes.c_int:
		return self.height

	@height.setter
	def height(self, i: ctypes.c_int) -> None:
		self.height = i

	@property
	def mipmaps(self) -> ctypes.c_int:
		return self.mipmaps

	@mipmaps.setter
	def mipmaps(self, i: ctypes.c_int) -> None:
		self.mipmaps = i

	@property
	def format(self) -> ctypes.c_int:
		return self.format

	@format.setter
	def format(self, i: ctypes.c_int) -> None:
		self.format = i


class Texture2D(ctypes.Structure):
	"""Texture2D, same as Texture"""
	_fields_ = [
		('id', ctypes.c_uint),  # OpenGL texture id
		('width', ctypes.c_int),  # Texture base width
		('height', ctypes.c_int),  # Texture base height
		('mipmaps', ctypes.c_int),  # Mipmap levels, 1 by default
		('format', ctypes.c_int)  # Data format (PixelFormat type)
	]

	@property
	def id(self) -> ctypes.c_uint:
		return self.id

	@id.setter
	def id(self, i: ctypes.c_uint) -> None:
		self.id = i

	@property
	def width(self) -> ctypes.c_int:
		return self.width

	@width.setter
	def width(self, i: ctypes.c_int) -> None:
		self.width = i

	@property
	def height(self) -> ctypes.c_int:
		return self.height

	@height.setter
	def height(self, i: ctypes.c_int) -> None:
		self.height = i

	@property
	def mipmaps(self) -> ctypes.c_int:
		return self.mipmaps

	@mipmaps.setter
	def mipmaps(self, i: ctypes.c_int) -> None:
		self.mipmaps = i

	@property
	def format(self) -> ctypes.c_int:
		return self.format

	@format.setter
	def format(self, i: ctypes.c_int) -> None:
		self.format = i


class TextureCubemap(ctypes.Structure):
	"""TextureCubemap, same as Texture"""
	_fields_ = [
		('id', ctypes.c_uint),  # OpenGL texture id
		('width', ctypes.c_int),  # Texture base width
		('height', ctypes.c_int),  # Texture base height
		('mipmaps', ctypes.c_int),  # Mipmap levels, 1 by default
		('format', ctypes.c_int)  # Data format (PixelFormat type)
	]

	@property
	def id(self) -> ctypes.c_uint:
		return self.id

	@id.setter
	def id(self, i: ctypes.c_uint) -> None:
		self.id = i

	@property
	def width(self) -> ctypes.c_int:
		return self.width

	@width.setter
	def width(self, i: ctypes.c_int) -> None:
		self.width = i

	@property
	def height(self) -> ctypes.c_int:
		return self.height

	@height.setter
	def height(self, i: ctypes.c_int) -> None:
		self.height = i

	@property
	def mipmaps(self) -> ctypes.c_int:
		return self.mipmaps

	@mipmaps.setter
	def mipmaps(self, i: ctypes.c_int) -> None:
		self.mipmaps = i

	@property
	def format(self) -> ctypes.c_int:
		return self.format

	@format.setter
	def format(self, i: ctypes.c_int) -> None:
		self.format = i


class RenderTexture(ctypes.Structure):
	"""RenderTexture, fbo for texture rendering"""
	_fields_ = [
		('id', ctypes.c_uint),  # OpenGL framebuffer object id
		('texture', Texture),  # Color buffer attachment texture
		('depth', Texture)  # Depth buffer attachment texture
	]

	@property
	def id(self) -> ctypes.c_uint:
		return self.id

	@id.setter
	def id(self, i: ctypes.c_uint) -> None:
		self.id = i

	@property
	def texture(self) -> Texture:
		return self.texture

	@texture.setter
	def texture(self, i: Texture) -> None:
		self.texture = i

	@property
	def depth(self) -> Texture:
		return self.depth

	@depth.setter
	def depth(self, i: Texture) -> None:
		self.depth = i


class RenderTexture2D(ctypes.Structure):
	"""RenderTexture2D, same as RenderTexture"""
	_fields_ = [
		('id', ctypes.c_uint),  # OpenGL framebuffer object id
		('texture', Texture),  # Color buffer attachment texture
		('depth', Texture)  # Depth buffer attachment texture
	]

	@property
	def id(self) -> ctypes.c_uint:
		return self.id

	@id.setter
	def id(self, i: ctypes.c_uint) -> None:
		self.id = i

	@property
	def texture(self) -> Texture:
		return self.texture

	@texture.setter
	def texture(self, i: Texture) -> None:
		self.texture = i

	@property
	def depth(self) -> Texture:
		return self.depth

	@depth.setter
	def depth(self, i: Texture) -> None:
		self.depth = i


class NPatchInfo(ctypes.Structure):
	"""NPatchInfo, n-patch layout info"""
	_fields_ = [
		('source', Rectangle),  # Texture source rectangle
		('left', ctypes.c_int),  # Left border offset
		('top', ctypes.c_int),  # Top border offset
		('right', ctypes.c_int),  # Right border offset
		('bottom', ctypes.c_int),  # Bottom border offset
		('layout', ctypes.c_int),  # Layout of the n-patch: 3x3 1x3 or 3x1
	]

	@property
	def source(self) -> Rectangle:
		return self.source

	@source.setter
	def source(self, i: Rectangle) -> None:
		self.source = i

	@property
	def left(self) -> ctypes.c_int:
		return self.left

	@left.setter
	def left(self, i: ctypes.c_int) -> None:
		self.left = i

	@property
	def top(self) -> ctypes.c_int:
		return self.top

	@top.setter
	def top(self, i: ctypes.c_int) -> None:
		self.top = i

	@property
	def right(self) -> ctypes.c_int:
		return self.right

	@right.setter
	def right(self, i: ctypes.c_int) -> None:
		self.right = i

	@property
	def bottom(self) -> ctypes.c_int:
		return self.bottom

	@bottom.setter
	def bottom(self, i: ctypes.c_int) -> None:
		self.bottom = i

	@property
	def layout(self) -> ctypes.c_int:
		return self.layout

	@layout.setter
	def layout(self, i: ctypes.c_int) -> None:
		self.layout = i


class GlyphInfo(ctypes.Structure):
	"""GlyphInfo, font characters glyphs info"""
	_fields_ = [
		('value', ctypes.c_int),  # Character value (Unicode)
		('offsetX', ctypes.c_int),  # Character offset X when drawing
		('offsetY', ctypes.c_int),  # Character offset Y when drawing
		('advanceX', ctypes.c_int),  # Character advance position X
		('image', Image)  # Character image data
	]

	@property
	def value(self) -> ctypes.c_int:
		return self.value

	@value.setter
	def value(self, i: ctypes.c_int) -> None:
		self.value = i

	@property
	def offsetX(self) -> ctypes.c_int:
		return self.offsetX

	@offsetX.setter
	def offsetX(self, i: ctypes.c_int) -> None:
		self.offsetX = i

	@property
	def offsetY(self) -> ctypes.c_int:
		return self.offsetY

	@offsetY.setter
	def offsetY(self, i: ctypes.c_int) -> None:
		self.offsetY = i

	@property
	def advanceX(self) -> ctypes.c_int:
		return self.advanceX

	@advanceX.setter
	def advanceX(self, i: ctypes.c_int) -> None:
		self.advanceX = i

	@property
	def image(self) -> Image:
		return self.image

	@image.setter
	def image(self, i: Image) -> None:
		self.image = i


class Font(ctypes.Structure):
	"""Font, font texture and GlyphInfo array data"""
	_fields_ = [
		('baseSize', ctypes.c_int),  # Base size (default chars height)
		('glyphCount', ctypes.c_int),  # Number of glyph characters
		('glyphPadding', ctypes.c_int),  # Padding around the glyph characters
		('texture', Texture2D),  # Texture atlas containing the glyphs
		('recs', ctypes.POINTER(Rectangle)),  # Rectangles in texture for the glyphs
		('glyphs', ctypes.POINTER(GlyphInfo))  # Glyphs info data
	]

	@property
	def baseSize(self) -> ctypes.c_int:
		return self.baseSize

	@baseSize.setter
	def baseSize(self, i: ctypes.c_int) -> None:
		self.baseSize = i

	@property
	def glyphCount(self) -> ctypes.c_int:
		return self.glyphCount

	@glyphCount.setter
	def glyphCount(self, i: ctypes.c_int) -> None:
		self.glyphCount = i

	@property
	def glyphPadding(self) -> ctypes.c_int:
		return self.glyphPadding

	@glyphPadding.setter
	def glyphPadding(self, i: ctypes.c_int) -> None:
		self.glyphPadding = i

	@property
	def texture(self) -> Texture2D:
		return self.texture

	@texture.setter
	def texture(self, i: Texture2D) -> None:
		self.texture = i

	@property
	def recs(self) -> ctypes.POINTER(Rectangle):
		return self.recs

	@recs.setter
	def recs(self, i: ctypes.POINTER(Rectangle)) -> None:
		self.recs = i

	@property
	def glyphs(self) -> ctypes.POINTER(GlyphInfo):
		return self.glyphs

	@glyphs.setter
	def glyphs(self, i: ctypes.POINTER(GlyphInfo)) -> None:
		self.glyphs = i


class Camera3D(ctypes.Structure):
	"""Camera, defines position/orientation in 3d space"""
	_fields_ = [
		('position', Vector3),  # Camera position
		('target', Vector3),  # Camera target it looks-at
		('up', Vector3),  # Camera up vector (rotation over its axis)
		('fovy', ctypes.c_float),  # Camera field-of-view apperture in Y (degrees) in perspective, used as near plane width in orthographic
		('projection', ctypes.c_int)  # Camera projection: CAMERA_PERSPECTIVE or CAMERA_ORTHOGRAPHIC
	]

	@property
	def position(self) -> Vector3:
		return self.position

	@position.setter
	def position(self, i: Vector3) -> None:
		self.position = i

	@property
	def target(self) -> Vector3:
		return self.target

	@target.setter
	def target(self, i: Vector3) -> None:
		self.target = i

	@property
	def up(self) -> Vector3:
		return self.up

	@up.setter
	def up(self, i: Vector3) -> None:
		self.up = i

	@property
	def fovy(self) -> ctypes.c_float:
		return self.fovy

	@fovy.setter
	def fovy(self, i: ctypes.c_float) -> None:
		self.fovy = i

	@property
	def projection(self) -> ctypes.c_int:
		return self.projection

	@projection.setter
	def projection(self, i: ctypes.c_int) -> None:
		self.projection = i


class Camera(ctypes.Structure):
	"""Camera type fallback, defaults to Camera3D"""
	_fields_ = [
		('position', Vector3),  # Camera position
		('target', Vector3),  # Camera target it looks-at
		('up', Vector3),  # Camera up vector (rotation over its axis)
		('fovy', ctypes.c_float),  # Camera field-of-view apperture in Y (degrees) in perspective, used as near plane width in orthographic
		('projection', ctypes.c_int)  # Camera projection: CAMERA_PERSPECTIVE or CAMERA_ORTHOGRAPHIC
	]

	@property
	def position(self) -> Vector3:
		return self.position

	@position.setter
	def position(self, i: Vector3) -> None:
		self.position = i

	@property
	def target(self) -> Vector3:
		return self.target

	@target.setter
	def target(self, i: Vector3) -> None:
		self.target = i

	@property
	def up(self) -> Vector3:
		return self.up

	@up.setter
	def up(self, i: Vector3) -> None:
		self.up = i

	@property
	def fovy(self) -> ctypes.c_float:
		return self.fovy

	@fovy.setter
	def fovy(self, i: ctypes.c_float) -> None:
		self.fovy = i

	@property
	def projection(self) -> ctypes.c_int:
		return self.projection

	@projection.setter
	def projection(self, i: ctypes.c_int) -> None:
		self.projection = i


class Camera2D(ctypes.Structure):
	"""Camera2D, defines position/orientation in 2d space"""
	_fields_ = [
		('offset', Vector2),  # Camera offset (displacement from target)
		('target', Vector2),  # Camera target (rotation and zoom origin)
		('rotation', ctypes.c_float),  # Camera rotation in degrees
		('zoom', ctypes.c_float),  # Camera zoom (scaling) should be 1.0f by default
	]

	@property
	def offset(self) -> Vector2:
		return self.offset

	@offset.setter
	def offset(self, i: Vector2) -> None:
		self.offset = i

	@property
	def target(self) -> Vector2:
		return self.target

	@target.setter
	def target(self, i: Vector2) -> None:
		self.target = i

	@property
	def rotation(self) -> ctypes.c_float:
		return self.rotation

	@rotation.setter
	def rotation(self, i: ctypes.c_float) -> None:
		self.rotation = i

	@property
	def zoom(self) -> ctypes.c_float:
		return self.zoom

	@zoom.setter
	def zoom(self, i: ctypes.c_float) -> None:
		self.zoom = i


class Mesh(ctypes.Structure):
	"""Mesh, vertex data and vao/vbo"""
	_fields_ = [
		('vertexCount', ctypes.c_int),  # Number of vertices stored in arrays
		('triangleCount', ctypes.c_int),  # Number of triangles stored (indexed or not)
		('vertices', ctypes.POINTER(ctypes.c_float)),  # Vertex position (XYZ - 3 components per vertex) (shader-location = 0)
		('texcoords', ctypes.POINTER(ctypes.c_float)),  # Vertex texture coordinates (UV - 2 components per vertex) (shader-location = 1)
		('texcoords2', ctypes.POINTER(ctypes.c_float)),  # Vertex texture second coordinates (UV - 2 components per vertex) (shader-location = 5)
		('normals', ctypes.POINTER(ctypes.c_float)),  # Vertex normals (XYZ - 3 components per vertex) (shader-location = 2)
		('tangents', ctypes.POINTER(ctypes.c_float)),  # Vertex tangents (XYZW - 4 components per vertex) (shader-location = 4)
		('colors', ctypes.POINTER(ctypes.c_ubyte)),  # Vertex colors (RGBA - 4 components per vertex) (shader-location = 3)
		('indices', ctypes.POINTER(ctypes.c_ushort)),  # Vertex indices (in case vertex data comes indexed)
		('animVertices', ctypes.POINTER(ctypes.c_float)),  # Animated vertex positions (after bones transformations)
		('animNormals', ctypes.POINTER(ctypes.c_float)),  # Animated normals (after bones transformations)
		('boneIds', ctypes.POINTER(ctypes.c_ubyte)),  # Vertex bone ids, max 255 bone ids, up to 4 bones influence by vertex (skinning)
		('boneWeights', ctypes.POINTER(ctypes.c_float)),  # Vertex bone weight, up to 4 bones influence by vertex (skinning)
		('vaoId', ctypes.c_uint),  # OpenGL Vertex Array Object id
		('vboId', ctypes.POINTER(ctypes.c_uint))  # OpenGL Vertex Buffer Objects id (default vertex data)
	]

	@property
	def vertexCount(self) -> ctypes.c_int:
		return self.vertexCount

	@vertexCount.setter
	def vertexCount(self, i: ctypes.c_int) -> None:
		self.vertexCount = i

	@property
	def triangleCount(self) -> ctypes.c_int:
		return self.triangleCount

	@triangleCount.setter
	def triangleCount(self, i: ctypes.c_int) -> None:
		self.triangleCount = i

	@property
	def vertices(self) -> ctypes.POINTER(ctypes.c_float):
		return self.vertices

	@vertices.setter
	def vertices(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.vertices = i

	@property
	def texcoords(self) -> ctypes.POINTER(ctypes.c_float):
		return self.texcoords

	@texcoords.setter
	def texcoords(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.texcoords = i

	@property
	def texcoords2(self) -> ctypes.POINTER(ctypes.c_float):
		return self.texcoords2

	@texcoords2.setter
	def texcoords2(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.texcoords2 = i

	@property
	def normals(self) -> ctypes.POINTER(ctypes.c_float):
		return self.normals

	@normals.setter
	def normals(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.normals = i

	@property
	def tangents(self) -> ctypes.POINTER(ctypes.c_float):
		return self.tangents

	@tangents.setter
	def tangents(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.tangents = i

	@property
	def colors(self) -> ctypes.POINTER(ctypes.c_ubyte):
		return self.colors

	@colors.setter
	def colors(self, i: ctypes.POINTER(ctypes.c_ubyte)) -> None:
		self.colors = i

	@property
	def indices(self) -> ctypes.POINTER(ctypes.c_ushort):
		return self.indices

	@indices.setter
	def indices(self, i: ctypes.POINTER(ctypes.c_ushort)) -> None:
		self.indices = i

	@property
	def animVertices(self) -> ctypes.POINTER(ctypes.c_float):
		return self.animVertices

	@animVertices.setter
	def animVertices(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.animVertices = i

	@property
	def animNormals(self) -> ctypes.POINTER(ctypes.c_float):
		return self.animNormals

	@animNormals.setter
	def animNormals(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.animNormals = i

	@property
	def boneIds(self) -> ctypes.POINTER(ctypes.c_ubyte):
		return self.boneIds

	@boneIds.setter
	def boneIds(self, i: ctypes.POINTER(ctypes.c_ubyte)) -> None:
		self.boneIds = i

	@property
	def boneWeights(self) -> ctypes.POINTER(ctypes.c_float):
		return self.boneWeights

	@boneWeights.setter
	def boneWeights(self, i: ctypes.POINTER(ctypes.c_float)) -> None:
		self.boneWeights = i

	@property
	def vaoId(self) -> ctypes.c_uint:
		return self.vaoId

	@vaoId.setter
	def vaoId(self, i: ctypes.c_uint) -> None:
		self.vaoId = i

	@property
	def vboId(self) -> ctypes.POINTER(ctypes.c_uint):
		return self.vboId

	@vboId.setter
	def vboId(self, i: ctypes.POINTER(ctypes.c_uint)) -> None:
		self.vboId = i


class Shader(ctypes.Structure):
	"""Shader"""
	_fields_ = [
		('id', ctypes.c_uint),  # Shader program id
		('locs', ctypes.POINTER(ctypes.c_int))  # Shader locations array (RL_MAX_SHADER_LOCATIONS)
	]

	@property
	def id(self) -> ctypes.c_uint:
		return self.id

	@id.setter
	def id(self, i: ctypes.c_uint) -> None:
		self.id = i

	@property
	def locs(self) -> ctypes.POINTER(ctypes.c_int):
		return self.locs

	@locs.setter
	def locs(self, i: ctypes.POINTER(ctypes.c_int)) -> None:
		self.locs = i


class MaterialMap(ctypes.Structure):
	"""MaterialMap"""
	_fields_ = [
		('texture', Texture2D),  # Material map texture
		('color', Color),  # Material map color
		('value', ctypes.c_float)  # Material map value
	]

	@property
	def texture(self) -> Texture2D:
		return self.texture

	@texture.setter
	def texture(self, i: Texture2D) -> None:
		self.texture = i

	@property
	def color(self) -> Color:
		return self.color

	@color.setter
	def color(self, i: Color) -> None:
		self.color = i

	@property
	def value(self) -> ctypes.c_float:
		return self.value

	@value.setter
	def value(self, i: ctypes.c_float) -> None:
		self.value = i


class Material(ctypes.Structure):
	"""Material, includes shader and maps"""
	_fields_ = [
		('shader', Shader),  # Material shader
		('maps', ctypes.POINTER(MaterialMap)),  # Material maps array (MAX_MATERIAL_MAPS)
		('params', ctypes.c_float * 4)  # Material generic parameters (if required)
	]

	@property
	def shader(self) -> Shader:
		return self.shader

	@shader.setter
	def shader(self, i: Shader) -> None:
		self.shader = i

	@property
	def maps(self) -> ctypes.POINTER(MaterialMap):
		return self.maps

	@maps.setter
	def maps(self, i: ctypes.POINTER(MaterialMap)) -> None:
		self.maps = i

	@property
	def params(self) -> ctypes.c_float * 4:
		return self.params

	@params.setter
	def params(self, i: ctypes.c_float * 4) -> None:
		self.params = i


class Transform(ctypes.Structure):
	"""Transform, vectex transformation data"""
	_fields_ = [
		('translation', Vector3),  # Translation
		('rotation', Quaternion),  # Rotation
		('scale', Vector3)  # Scale
	]

	@property
	def translation(self) -> Vector3:
		return self.translation

	@translation.setter
	def translation(self, i: Vector3) -> None:
		self.translation = i

	@property
	def rotation(self) -> Quaternion:
		return self.rotation

	@rotation.setter
	def rotation(self, i: Quaternion) -> None:
		self.rotation = i

	@property
	def scale(self) -> Vector3:
		return self.scale

	@scale.setter
	def scale(self, i: Vector3) -> None:
		self.scale = i


class BoneInfo(ctypes.Structure):
	"""Bone, skeletal animation bone"""
	_fields_ = [
		('name', ctypes.c_char * 32),  # Bone name
		('parent', ctypes.c_int)  # Bone parent
	]

	@property
	def name(self) -> ctypes.c_char * 32:
		return self.name

	@name.setter
	def name(self, i: ctypes.c_char * 32) -> None:
		self.name = i

	@property
	def parent(self) -> ctypes.c_int:
		return self.parent

	@parent.setter
	def parent(self, i: ctypes.c_int) -> None:
		self.parent = i


class Model(ctypes.Structure):
	"""Model, meshes, materials and animation data"""
	_fields_ = [
		('transform', Matrix),  # Local transform matrix
		('meshCount', ctypes.c_int),  # Number of meshes
		('materialCount', ctypes.c_int),  # Number of materials
		('meshes', ctypes.POINTER(Mesh)),  # Meshes array
		('materials', ctypes.POINTER(Material)),  # Materials array
		('meshMaterial', ctypes.POINTER(ctypes.c_int)),  # Mesh material number
		('boneCount', ctypes.c_int),  # Number of bones
		('bones', ctypes.POINTER(BoneInfo)),  # Bones information (skeleton)
		('bindPose', ctypes.POINTER(Transform))  # Bones base transformation (pose)
	]

	@property
	def transform(self) -> Matrix:
		return self.transform

	@transform.setter
	def transform(self, i: Matrix) -> None:
		self.transform = i

	@property
	def meshCount(self) -> ctypes.c_int:
		return self.meshCount

	@meshCount.setter
	def meshCount(self, i: ctypes.c_int) -> None:
		self.meshCount = i

	@property
	def materialCount(self) -> ctypes.c_int:
		return self.materialCount

	@materialCount.setter
	def materialCount(self, i: ctypes.c_int) -> None:
		self.materialCount = i

	@property
	def meshes(self) -> ctypes.POINTER(Mesh):
		return self.meshes

	@meshes.setter
	def meshes(self, i: ctypes.POINTER(Mesh)) -> None:
		self.meshes = i

	@property
	def materials(self) -> ctypes.POINTER(Material):
		return self.materials

	@materials.setter
	def materials(self, i: ctypes.POINTER(Material)) -> None:
		self.materials = i

	@property
	def meshMaterial(self) -> ctypes.POINTER(ctypes.c_int):
		return self.meshMaterial

	@meshMaterial.setter
	def meshMaterial(self, i: ctypes.POINTER(ctypes.c_int)) -> None:
		self.meshMaterial = i

	@property
	def boneCount(self) -> ctypes.c_int:
		return self.boneCount

	@boneCount.setter
	def boneCount(self, i: ctypes.c_int) -> None:
		self.boneCount = i

	@property
	def bones(self) -> ctypes.POINTER(BoneInfo):
		return self.bones

	@bones.setter
	def bones(self, i: ctypes.POINTER(BoneInfo)) -> None:
		self.bones = i

	@property
	def bindPose(self) -> ctypes.POINTER(Transform):
		return self.bindPose

	@bindPose.setter
	def bindPose(self, i: ctypes.POINTER(Transform)) -> None:
		self.bindPose = i


class ModelAnimation(ctypes.Structure):
	"""ModelAnimation"""
	_fields_ = [
		('boneCount', ctypes.c_int),  # Number of bones
		('frameCount', ctypes.c_int),  # Number of animation frames
		('bones', ctypes.POINTER(BoneInfo)),  # Bones information (skeleton)
		('framePoses', ctypes.POINTER(ctypes.POINTER(Transform)))  # Poses array by frame
	]

	@property
	def boneCount(self) -> ctypes.c_int:
		return self.boneCount

	@boneCount.setter
	def boneCount(self, i: ctypes.c_int) -> None:
		self.boneCount = i

	@property
	def frameCount(self) -> ctypes.c_int:
		return self.frameCount

	@frameCount.setter
	def frameCount(self, i: ctypes.c_int) -> None:
		self.frameCount = i

	@property
	def bones(self) -> ctypes.POINTER(BoneInfo):
		return self.bones

	@bones.setter
	def bones(self, i: ctypes.POINTER(BoneInfo)) -> None:
		self.bones = i

	@property
	def framePoses(self) -> ctypes.POINTER(ctypes.POINTER(Transform)):
		return self.framePoses

	@framePoses.setter
	def framePoses(self, i: ctypes.POINTER(ctypes.POINTER(Transform))) -> None:
		self.framePoses = i


class Ray(ctypes.Structure):
	"""Ray, ray for raycasting"""
	_fields_ = [
		('position', Vector3),  # Ray position (origin)
		('direction', Vector3)  # Ray direction
	]

	@property
	def position(self) -> Vector3:
		return self.position

	@position.setter
	def position(self, i: Vector3) -> None:
		self.position = i

	@property
	def direction(self) -> Vector3:
		return self.direction

	@direction.setter
	def direction(self, i: Vector3) -> None:
		self.direction = i


class RayCollision(ctypes.Structure):
	"""RayCollision, ray hit information"""
	_fields_ = [
		('hit', ctypes.c_bool),  # Did the ray hit something?
		('distance', ctypes.c_float),  # Distance to nearest hit
		('point', Vector3),  # Point of nearest hit
		('normal', Vector3)  # Surface normal of hit
	]

	@property
	def hit(self) -> ctypes.c_bool:
		return self.hit

	@hit.setter
	def hit(self, i: ctypes.c_bool) -> None:
		self.hit = i

	@property
	def distance(self) -> ctypes.c_float:
		return self.distance

	@distance.setter
	def distance(self, i: ctypes.c_float) -> None:
		self.distance = i

	@property
	def point(self) -> Vector3:
		return self.point

	@point.setter
	def point(self, i: Vector3) -> None:
		self.point = i

	@property
	def normal(self) -> Vector3:
		return self.normal

	@normal.setter
	def normal(self, i: Vector3) -> None:
		self.normal = i


class BoundingBox(ctypes.Structure):
	"""BoundingBox"""
	_fields_ = [
		('min', Vector3),  # Minimum vertex box-corner
		('max', Vector3)  # Maximum vertex box-corner
	]

	@property
	def min(self) -> Vector3:
		return self.min

	@min.setter
	def min(self, i: Vector3) -> None:
		self.min = i

	@property
	def max(self) -> Vector3:
		return self.max

	@max.setter
	def max(self, i: Vector3) -> None:
		self.max = i


class Wave(ctypes.Structure):
	"""Wave, audio wave data"""
	_fields_ = [
		('frameCount', ctypes.c_uint),  # Total number of frames (considering channels)
		('sampleRate', ctypes.c_uint),  # Frequency (samples per second)
		('sampleSize', ctypes.c_uint),  # Bit depth (bits per sample): 8, 16, 32 (24 not supported)
		('channels', ctypes.c_uint),  # Number of channels (1-mono, 2-stereo, ...)
		('data', ctypes.c_void_p)  # Buffer data pointer
	]

	@property
	def frameCount(self) -> ctypes.c_uint:
		return self.frameCount

	@frameCount.setter
	def frameCount(self, i: ctypes.c_uint) -> None:
		self.frameCount = i

	@property
	def sampleRate(self) -> ctypes.c_uint:
		return self.sampleRate

	@sampleRate.setter
	def sampleRate(self, i: ctypes.c_uint) -> None:
		self.sampleRate = i

	@property
	def sampleSize(self) -> ctypes.c_uint:
		return self.sampleSize

	@sampleSize.setter
	def sampleSize(self, i: ctypes.c_uint) -> None:
		self.sampleSize = i

	@property
	def channels(self) -> ctypes.c_uint:
		return self.channels

	@channels.setter
	def channels(self, i: ctypes.c_uint) -> None:
		self.channels = i

	@property
	def data(self) -> ctypes.c_void_p:
		return self.data

	@data.setter
	def data(self, i: ctypes.c_void_p) -> None:
		self.data = i


class AudioStream(ctypes.Structure):
	"""AudioStream, custom audio stream"""
	_fields_ = [
		('buffer', ctypes.POINTER(rAudioBuffer)),  # Pointer to internal data used by the audio system
		('processor', ctypes.POINTER(rAudioProcessor)),  # Pointer to internal data processor, useful for audio effects
		('sampleRate', ctypes.c_uint),  # Frequency (samples per second)
		('sampleSize', ctypes.c_uint),  # Bit depth (bits per sample): 8, 16, 32 (24 not supported)
		('channels', ctypes.c_uint),  # Number of channels (1-mono, 2-stereo ...)
	]

	@property
	def buffer(self) -> ctypes.POINTER(rAudioBuffer):
		return self.buffer

	@buffer.setter
	def buffer(self, i: ctypes.POINTER(rAudioBuffer)) -> None:
		self.buffer = i

	@property
	def processor(self) -> ctypes.POINTER(rAudioProcessor):
		return self.processor

	@processor.setter
	def processor(self, i: ctypes.POINTER(rAudioProcessor)) -> None:
		self.processor = i

	@property
	def sampleRate(self) -> ctypes.c_uint:
		return self.sampleRate

	@sampleRate.setter
	def sampleRate(self, i: ctypes.c_uint) -> None:
		self.sampleRate = i

	@property
	def sampleSize(self) -> ctypes.c_uint:
		return self.sampleSize

	@sampleSize.setter
	def sampleSize(self, i: ctypes.c_uint) -> None:
		self.sampleSize = i

	@property
	def channels(self) -> ctypes.c_uint:
		return self.channels

	@channels.setter
	def channels(self, i: ctypes.c_uint) -> None:
		self.channels = i


class Sound(ctypes.Structure):
	"""Sound"""
	_fields_ = [
		('stream', AudioStream),  # Audio stream
		('frameCount', ctypes.c_uint)  # Total number of frames (considering channels)
	]

	@property
	def stream(self) -> AudioStream:
		return self.stream

	@stream.setter
	def stream(self, i: AudioStream) -> None:
		self.stream = i

	@property
	def frameCount(self) -> ctypes.c_uint:
		return self.frameCount

	@frameCount.setter
	def frameCount(self, i: ctypes.c_uint) -> None:
		self.frameCount = i


class Music(ctypes.Structure):
	"""Music, audio stream, anything longer than ~10 seconds should be streamed"""
	_fields_ = [
		('stream', AudioStream),  # Audio stream
		('frameCount', ctypes.c_uint),  # Total number of frames (considering channels)
		('looping', ctypes.c_bool),  # Music looping enable
		('ctxType', ctypes.c_int),  # Type of music context (audio filetype)
		('ctxData', ctypes.c_void_p),  # Audio context data depends on type
	]

	@property
	def stream(self) -> AudioStream:
		return self.stream

	@stream.setter
	def stream(self, i: AudioStream) -> None:
		self.stream = i

	@property
	def frameCount(self) -> ctypes.c_uint:
		return self.frameCount

	@frameCount.setter
	def frameCount(self, i: ctypes.c_uint) -> None:
		self.frameCount = i

	@property
	def looping(self) -> ctypes.c_bool:
		return self.looping

	@looping.setter
	def looping(self, i: ctypes.c_bool) -> None:
		self.looping = i

	@property
	def ctxType(self) -> ctypes.c_int:
		return self.ctxType

	@ctxType.setter
	def ctxType(self, i: ctypes.c_int) -> None:
		self.ctxType = i

	@property
	def ctxData(self) -> ctypes.c_void_p:
		return self.ctxData

	@ctxData.setter
	def ctxData(self, i: ctypes.c_void_p) -> None:
		self.ctxData = i


class VrDeviceInfo(ctypes.Structure):
	"""VrDeviceInfo, Head-Mounted-Display device parameters"""
	_fields_ = [
		('hResolution', ctypes.c_int),  # Horizontal resolution in pixels
		('vResolution', ctypes.c_int),  # Vertical resolution in pixels
		('hScreenSize', ctypes.c_float),  # Horizontal size in meters
		('vScreenSize', ctypes.c_float),  # Vertical size in meters
		('vScreenCenter', ctypes.c_float),  # Screen center in meters
		('eyeToScreenDistance', ctypes.c_float),  # Distance between eye and display in meters
		('lensSeparationDistance', ctypes.c_float),  # Lens separation distance in meters
		('interpupillaryDistance', ctypes.c_float),  # IPD (distance between pupils) in meters
		('lensDistortionValues', ctypes.c_float * 4),  # Lens distortion constant parameters
		('chromaAbCorrection', ctypes.c_float * 4)  # Chromatic aberration correction parameters
	]

	@property
	def hResolution(self) -> ctypes.c_int:
		return self.hResolution

	@hResolution.setter
	def hResolution(self, i: ctypes.c_int) -> None:
		self.hResolution = i

	@property
	def vResolution(self) -> ctypes.c_int:
		return self.vResolution

	@vResolution.setter
	def vResolution(self, i: ctypes.c_int) -> None:
		self.vResolution = i

	@property
	def hScreenSize(self) -> ctypes.c_float:
		return self.hScreenSize

	@hScreenSize.setter
	def hScreenSize(self, i: ctypes.c_float) -> None:
		self.hScreenSize = i

	@property
	def vScreenSize(self) -> ctypes.c_float:
		return self.vScreenSize

	@vScreenSize.setter
	def vScreenSize(self, i: ctypes.c_float) -> None:
		self.vScreenSize = i

	@property
	def vScreenCenter(self) -> ctypes.c_float:
		return self.vScreenCenter

	@vScreenCenter.setter
	def vScreenCenter(self, i: ctypes.c_float) -> None:
		self.vScreenCenter = i

	@property
	def eyeToScreenDistance(self) -> ctypes.c_float:
		return self.eyeToScreenDistance

	@eyeToScreenDistance.setter
	def eyeToScreenDistance(self, i: ctypes.c_float) -> None:
		self.eyeToScreenDistance = i

	@property
	def lensSeparationDistance(self) -> ctypes.c_float:
		return self.lensSeparationDistance

	@lensSeparationDistance.setter
	def lensSeparationDistance(self, i: ctypes.c_float) -> None:
		self.lensSeparationDistance = i

	@property
	def interpupillaryDistance(self) -> ctypes.c_float:
		return self.interpupillaryDistance

	@interpupillaryDistance.setter
	def interpupillaryDistance(self, i: ctypes.c_float) -> None:
		self.interpupillaryDistance = i

	@property
	def lensDistortionValues(self) -> ctypes.c_float * 4:
		return self.lensDistortionValues

	@lensDistortionValues.setter
	def lensDistortionValues(self, i: ctypes.c_float * 4) -> None:
		self.lensDistortionValues = i

	@property
	def chromaAbCorrection(self) -> ctypes.c_float * 4:
		return self.chromaAbCorrection

	@chromaAbCorrection.setter
	def chromaAbCorrection(self, i: ctypes.c_float * 4) -> None:
		self.chromaAbCorrection = i


class VrStereoConfig(ctypes.Structure):
	"""VrStereoConfig, VR stereo rendering configuration for simulator"""
	_fields_ = [
		('projection', Matrix * 2),  # VR projection matrices (per eye)
		('viewOffset', Matrix * 2),  # VR view offset matrices (per eye)
		('leftLensCenter', ctypes.c_float * 2),  # VR left lens center
		('rightLensCenter', ctypes.c_float * 2),  # VR right lens center
		('leftScreenCenter', ctypes.c_float * 2),  # VR left screen center
		('rightScreenCenter', ctypes.c_float * 2),  # VR right screen center
		('scale', ctypes.c_float * 2),  # VR distortion scale
		('scaleIn', ctypes.c_float * 2)  # VR distortion scale in
	]

	@property
	def projection(self) -> Matrix * 2:
		return self.projection

	@projection.setter
	def projection(self, i: Matrix * 2) -> None:
		self.projection = i

	@property
	def viewOffset(self) -> Matrix * 2:
		return self.viewOffset

	@viewOffset.setter
	def viewOffset(self, i: Matrix * 2) -> None:
		self.viewOffset = i

	@property
	def leftLensCenter(self) -> ctypes.c_float * 2:
		return self.leftLensCenter

	@leftLensCenter.setter
	def leftLensCenter(self, i: ctypes.c_float * 2) -> None:
		self.leftLensCenter = i

	@property
	def rightLensCenter(self) -> ctypes.c_float * 2:
		return self.rightLensCenter

	@rightLensCenter.setter
	def rightLensCenter(self, i: ctypes.c_float * 2) -> None:
		self.rightLensCenter = i

	@property
	def leftScreenCenter(self) -> ctypes.c_float * 2:
		return self.leftScreenCenter

	@leftScreenCenter.setter
	def leftScreenCenter(self, i: ctypes.c_float * 2) -> None:
		self.leftScreenCenter = i

	@property
	def rightScreenCenter(self) -> ctypes.c_float * 2:
		return self.rightScreenCenter

	@rightScreenCenter.setter
	def rightScreenCenter(self, i: ctypes.c_float * 2) -> None:
		self.rightScreenCenter = i

	@property
	def scale(self) -> ctypes.c_float * 2:
		return self.scale

	@scale.setter
	def scale(self, i: ctypes.c_float * 2) -> None:
		self.scale = i

	@property
	def scaleIn(self) -> ctypes.c_float * 2:
		return self.scaleIn

	@scaleIn.setter
	def scaleIn(self, i: ctypes.c_float * 2) -> None:
		self.scaleIn = i


class FilePathList(ctypes.Structure):
	"""File path list"""
	_fields_ = [
		('capacity', ctypes.c_uint),  # Filepaths max entries
		('count', ctypes.c_uint),  # Filepaths entries count
		('paths', ctypes.POINTER(ctypes.POINTER(ctypes.c_char)))  # Filepaths entries
	]

	@property
	def capacity(self) -> ctypes.c_uint:
		return self.capacity

	@capacity.setter
	def capacity(self, i: ctypes.c_uint) -> None:
		self.capacity = i

	@property
	def count(self) -> ctypes.c_uint:
		return self.count

	@count.setter
	def count(self, i: ctypes.c_uint) -> None:
		self.count = i

	@property
	def paths(self) -> ctypes.POINTER(ctypes.POINTER(ctypes.c_char)):
		return self.paths

	@paths.setter
	def paths(self, i: ctypes.POINTER(ctypes.POINTER(ctypes.c_char))) -> None:
		self.paths = i


class float3(ctypes.Structure):
	"""NOTE: Helper types to be used instead of array return types for *ToFloat functions"""
	_fields_ = [
		('v', ctypes.c_float * 3)  # 
	]

	@property
	def v(self) -> ctypes.c_float * 3:
		return self.v

	@v.setter
	def v(self, i: ctypes.c_float * 3) -> None:
		self.v = i


class float16(ctypes.Structure):
	""""""
	_fields_ = [
		('v', ctypes.c_float * 16)  # 
	]

	@property
	def v(self) -> ctypes.c_float * 16:
		return self.v

	@v.setter
	def v(self, i: ctypes.c_float * 16) -> None:
		self.v = i


class GuiStyleProp(ctypes.Structure):
	"""Style property"""
	_fields_ = [
		('controlId', ctypes.c_ushort),  # 
		('propertyId', ctypes.c_ushort),  # 
		('propertyValue', ctypes.c_uint)  # 
	]

	@property
	def controlId(self) -> ctypes.c_ushort:
		return self.controlId

	@controlId.setter
	def controlId(self, i: ctypes.c_ushort) -> None:
		self.controlId = i

	@property
	def propertyId(self) -> ctypes.c_ushort:
		return self.propertyId

	@propertyId.setter
	def propertyId(self, i: ctypes.c_ushort) -> None:
		self.propertyId = i

	@property
	def propertyValue(self) -> ctypes.c_uint:
		return self.propertyValue

	@propertyValue.setter
	def propertyValue(self, i: ctypes.c_uint) -> None:
		self.propertyValue = i


__structs = {
	"rlVertexBuffer": rlVertexBuffer,
	"rlDrawCall": rlDrawCall,
	"rlRenderBatch": rlRenderBatch,
	"Matrix": Matrix,
	"rlglData": rlglData,
	"Vector2": Vector2,
	"Vector3": Vector3,
	"Vector4": Vector4,
	"Quaternion": Quaternion,
	"Color": Color,
	"Rectangle": Rectangle,
	"Image": Image,
	"Texture": Texture,
	"Texture2D": Texture2D,
	"TextureCubemap": TextureCubemap,
	"RenderTexture": RenderTexture,
	"RenderTexture2D": RenderTexture2D,
	"NPatchInfo": NPatchInfo,
	"GlyphInfo": GlyphInfo,
	"Font": Font,
	"Camera3D": Camera3D,
	"Camera": Camera,
	"Camera2D": Camera2D,
	"Mesh": Mesh,
	"Shader": Shader,
	"MaterialMap": MaterialMap,
	"Material": Material,
	"Transform": Transform,
	"BoneInfo": BoneInfo,
	"Model": Model,
	"ModelAnimation": ModelAnimation,
	"Ray": Ray,
	"RayCollision": RayCollision,
	"BoundingBox": BoundingBox,
	"Wave": Wave,
	"AudioStream": AudioStream,
	"Sound": Sound,
	"Music": Music,
	"VrDeviceInfo": VrDeviceInfo,
	"VrStereoConfig": VrStereoConfig,
	"FilePathList": FilePathList,
	"float3": float3,
	"float16": float16,
	"GuiStyleProp": GuiStyleProp
}


from enum import IntEnum

class rlGlVersion(IntEnum):
	""""""
	OPENGL_11: int = 1  # 
	OPENGL_21: int = 2  # 
	OPENGL_33: int = 3  # 
	OPENGL_43: int = 4  # 
	OPENGL_ES_20: int = 5  # 


class rlFramebufferAttachType(IntEnum):
	""""""
	RL_ATTACHMENT_COLOR_CHANNEL0: int = 0  # 
	RL_ATTACHMENT_COLOR_CHANNEL1: int = 1  # 
	RL_ATTACHMENT_COLOR_CHANNEL2: int = 2  # 
	RL_ATTACHMENT_COLOR_CHANNEL3: int = 3  # 
	RL_ATTACHMENT_COLOR_CHANNEL4: int = 4  # 
	RL_ATTACHMENT_COLOR_CHANNEL5: int = 5  # 
	RL_ATTACHMENT_COLOR_CHANNEL6: int = 6  # 
	RL_ATTACHMENT_COLOR_CHANNEL7: int = 7  # 
	RL_ATTACHMENT_DEPTH: int = 100  # 
	RL_ATTACHMENT_STENCIL: int = 200  # 


class rlFramebufferAttachTextureType(IntEnum):
	""""""
	RL_ATTACHMENT_CUBEMAP_POSITIVE_X: int = 0  # 
	RL_ATTACHMENT_CUBEMAP_NEGATIVE_X: int = 1  # 
	RL_ATTACHMENT_CUBEMAP_POSITIVE_Y: int = 2  # 
	RL_ATTACHMENT_CUBEMAP_NEGATIVE_Y: int = 3  # 
	RL_ATTACHMENT_CUBEMAP_POSITIVE_Z: int = 4  # 
	RL_ATTACHMENT_CUBEMAP_NEGATIVE_Z: int = 5  # 
	RL_ATTACHMENT_TEXTURE2D: int = 100  # 
	RL_ATTACHMENT_RENDERBUFFER: int = 200  # 


class rlTraceLogLevel(IntEnum):
	"""Trace log level"""
	RL_LOG_ALL: int = 0  # Display all logs
	RL_LOG_TRACE: int = 1  # Trace logging, intended for internal use only
	RL_LOG_DEBUG: int = 2  # Debug logging, used for internal debugging, it should be disabled on release builds
	RL_LOG_INFO: int = 3  # Info logging, used for program execution info
	RL_LOG_WARNING: int = 4  # Warning logging, used on recoverable failures
	RL_LOG_ERROR: int = 5  # Error logging, used on unrecoverable failures
	RL_LOG_FATAL: int = 6  # Fatal logging, used to abort program: exit(EXIT_FAILURE)
	RL_LOG_NONE: int = 7  # Disable logging


class rlPixelFormat(IntEnum):
	"""Texture formats (support depends on OpenGL version)"""
	RL_PIXELFORMAT_UNCOMPRESSED_GRAYSCALE: int = 1  # 8 bit per pixel (no alpha)
	RL_PIXELFORMAT_UNCOMPRESSED_GRAY_ALPHA: int = 2  # 8*2 bpp (2 channels)
	RL_PIXELFORMAT_UNCOMPRESSED_R5G6B5: int = 3  # 16 bpp
	RL_PIXELFORMAT_UNCOMPRESSED_R8G8B8: int = 4  # 24 bpp
	RL_PIXELFORMAT_UNCOMPRESSED_R5G5B5A1: int = 5  # 16 bpp (1 bit alpha)
	RL_PIXELFORMAT_UNCOMPRESSED_R4G4B4A4: int = 6  # 16 bpp (4 bit alpha)
	RL_PIXELFORMAT_UNCOMPRESSED_R8G8B8A8: int = 7  # 32 bpp
	RL_PIXELFORMAT_UNCOMPRESSED_R32: int = 8  # 32 bpp (1 channel - float)
	RL_PIXELFORMAT_UNCOMPRESSED_R32G32B32: int = 9  # 32*3 bpp (3 channels - float)
	RL_PIXELFORMAT_UNCOMPRESSED_R32G32B32A32: int = 10  # 32*4 bpp (4 channels - float)
	RL_PIXELFORMAT_COMPRESSED_DXT1_RGB: int = 11  # 4 bpp (no alpha)
	RL_PIXELFORMAT_COMPRESSED_DXT1_RGBA: int = 12  # 4 bpp (1 bit alpha)
	RL_PIXELFORMAT_COMPRESSED_DXT3_RGBA: int = 13  # 8 bpp
	RL_PIXELFORMAT_COMPRESSED_DXT5_RGBA: int = 14  # 8 bpp
	RL_PIXELFORMAT_COMPRESSED_ETC1_RGB: int = 15  # 4 bpp
	RL_PIXELFORMAT_COMPRESSED_ETC2_RGB: int = 16  # 4 bpp
	RL_PIXELFORMAT_COMPRESSED_ETC2_EAC_RGBA: int = 17  # 8 bpp
	RL_PIXELFORMAT_COMPRESSED_PVRT_RGB: int = 18  # 4 bpp
	RL_PIXELFORMAT_COMPRESSED_PVRT_RGBA: int = 19  # 4 bpp
	RL_PIXELFORMAT_COMPRESSED_ASTC_4x4_RGBA: int = 20  # 8 bpp
	RL_PIXELFORMAT_COMPRESSED_ASTC_8x8_RGBA: int = 21  # 2 bpp


class rlTextureFilter(IntEnum):
	"""Texture parameters: filter mode"""
	RL_TEXTURE_FILTER_POINT: int = 0  # No filter, just pixel approximation
	RL_TEXTURE_FILTER_BILINEAR: int = 1  # Linear filtering
	RL_TEXTURE_FILTER_TRILINEAR: int = 2  # Trilinear filtering (linear with mipmaps)
	RL_TEXTURE_FILTER_ANISOTROPIC_4X: int = 3  # Anisotropic filtering 4x
	RL_TEXTURE_FILTER_ANISOTROPIC_8X: int = 4  # Anisotropic filtering 8x
	RL_TEXTURE_FILTER_ANISOTROPIC_16X: int = 5  # Anisotropic filtering 16x


class rlBlendMode(IntEnum):
	"""Color blending modes (pre-defined)"""
	RL_BLEND_ALPHA: int = 0  # Blend textures considering alpha (default)
	RL_BLEND_ADDITIVE: int = 1  # Blend textures adding colors
	RL_BLEND_MULTIPLIED: int = 2  # Blend textures multiplying colors
	RL_BLEND_ADD_COLORS: int = 3  # Blend textures adding colors (alternative)
	RL_BLEND_SUBTRACT_COLORS: int = 4  # Blend textures subtracting colors (alternative)
	RL_BLEND_ALPHA_PREMULTIPLY: int = 5  # Blend premultiplied textures considering alpha
	RL_BLEND_CUSTOM: int = 6  # Blend textures using custom src/dst factors (use rlSetBlendFactors())


class rlShaderLocationIndex(IntEnum):
	"""Shader location point type"""
	RL_SHADER_LOC_VERTEX_POSITION: int = 0  # Shader location: vertex attribute: position
	RL_SHADER_LOC_VERTEX_TEXCOORD01: int = 1  # Shader location: vertex attribute: texcoord01
	RL_SHADER_LOC_VERTEX_TEXCOORD02: int = 2  # Shader location: vertex attribute: texcoord02
	RL_SHADER_LOC_VERTEX_NORMAL: int = 3  # Shader location: vertex attribute: normal
	RL_SHADER_LOC_VERTEX_TANGENT: int = 4  # Shader location: vertex attribute: tangent
	RL_SHADER_LOC_VERTEX_COLOR: int = 5  # Shader location: vertex attribute: color
	RL_SHADER_LOC_MATRIX_MVP: int = 6  # Shader location: matrix uniform: model-view-projection
	RL_SHADER_LOC_MATRIX_VIEW: int = 7  # Shader location: matrix uniform: view (camera transform)
	RL_SHADER_LOC_MATRIX_PROJECTION: int = 8  # Shader location: matrix uniform: projection
	RL_SHADER_LOC_MATRIX_MODEL: int = 9  # Shader location: matrix uniform: model (transform)
	RL_SHADER_LOC_MATRIX_NORMAL: int = 10  # Shader location: matrix uniform: normal
	RL_SHADER_LOC_VECTOR_VIEW: int = 11  # Shader location: vector uniform: view
	RL_SHADER_LOC_COLOR_DIFFUSE: int = 12  # Shader location: vector uniform: diffuse color
	RL_SHADER_LOC_COLOR_SPECULAR: int = 13  # Shader location: vector uniform: specular color
	RL_SHADER_LOC_COLOR_AMBIENT: int = 14  # Shader location: vector uniform: ambient color
	RL_SHADER_LOC_MAP_ALBEDO: int = 15  # Shader location: sampler2d texture: albedo (same as: RL_SHADER_LOC_MAP_DIFFUSE)
	RL_SHADER_LOC_MAP_METALNESS: int = 16  # Shader location: sampler2d texture: metalness (same as: RL_SHADER_LOC_MAP_SPECULAR)
	RL_SHADER_LOC_MAP_NORMAL: int = 17  # Shader location: sampler2d texture: normal
	RL_SHADER_LOC_MAP_ROUGHNESS: int = 18  # Shader location: sampler2d texture: roughness
	RL_SHADER_LOC_MAP_OCCLUSION: int = 19  # Shader location: sampler2d texture: occlusion
	RL_SHADER_LOC_MAP_EMISSION: int = 20  # Shader location: sampler2d texture: emission
	RL_SHADER_LOC_MAP_HEIGHT: int = 21  # Shader location: sampler2d texture: height
	RL_SHADER_LOC_MAP_CUBEMAP: int = 22  # Shader location: samplerCube texture: cubemap
	RL_SHADER_LOC_MAP_IRRADIANCE: int = 23  # Shader location: samplerCube texture: irradiance
	RL_SHADER_LOC_MAP_PREFILTER: int = 24  # Shader location: samplerCube texture: prefilter
	RL_SHADER_LOC_MAP_BRDF: int = 25  # Shader location: sampler2d texture: brdf


class rlShaderUniformDataType(IntEnum):
	"""Shader uniform data type"""
	RL_SHADER_UNIFORM_FLOAT: int = 0  # Shader uniform type: float
	RL_SHADER_UNIFORM_VEC2: int = 1  # Shader uniform type: vec2 (2 float)
	RL_SHADER_UNIFORM_VEC3: int = 2  # Shader uniform type: vec3 (3 float)
	RL_SHADER_UNIFORM_VEC4: int = 3  # Shader uniform type: vec4 (4 float)
	RL_SHADER_UNIFORM_INT: int = 4  # Shader uniform type: int
	RL_SHADER_UNIFORM_IVEC2: int = 5  # Shader uniform type: ivec2 (2 int)
	RL_SHADER_UNIFORM_IVEC3: int = 6  # Shader uniform type: ivec3 (3 int)
	RL_SHADER_UNIFORM_IVEC4: int = 7  # Shader uniform type: ivec4 (4 int)
	RL_SHADER_UNIFORM_SAMPLER2D: int = 8  # Shader uniform type: sampler2d


class rlShaderAttributeDataType(IntEnum):
	"""Shader attribute data types"""
	RL_SHADER_ATTRIB_FLOAT: int = 0  # Shader attribute type: float
	RL_SHADER_ATTRIB_VEC2: int = 1  # Shader attribute type: vec2 (2 float)
	RL_SHADER_ATTRIB_VEC3: int = 2  # Shader attribute type: vec3 (3 float)
	RL_SHADER_ATTRIB_VEC4: int = 3  # Shader attribute type: vec4 (4 float)


class ConfigFlags(IntEnum):
	"""System/Window config flags"""
	FLAG_VSYNC_HINT: int = 64  # Set to try enabling V-Sync on GPU
	FLAG_FULLSCREEN_MODE: int = 2  # Set to run program in fullscreen
	FLAG_WINDOW_RESIZABLE: int = 4  # Set to allow resizable window
	FLAG_WINDOW_UNDECORATED: int = 8  # Set to disable window decoration (frame and buttons)
	FLAG_WINDOW_HIDDEN: int = 128  # Set to hide window
	FLAG_WINDOW_MINIMIZED: int = 512  # Set to minimize window (iconify)
	FLAG_WINDOW_MAXIMIZED: int = 1024  # Set to maximize window (expanded to monitor)
	FLAG_WINDOW_UNFOCUSED: int = 2048  # Set to window non focused
	FLAG_WINDOW_TOPMOST: int = 4096  # Set to window always on top
	FLAG_WINDOW_ALWAYS_RUN: int = 256  # Set to allow windows running while minimized
	FLAG_WINDOW_TRANSPARENT: int = 16  # Set to allow transparent framebuffer
	FLAG_WINDOW_HIGHDPI: int = 8192  # Set to support HighDPI
	FLAG_WINDOW_MOUSE_PASSTHROUGH: int = 16384  # Set to support mouse passthrough, only supported when FLAG_WINDOW_UNDECORATED
	FLAG_MSAA_4X_HINT: int = 32  # Set to try enabling MSAA 4X
	FLAG_INTERLACED_HINT: int = 65536  # Set to try enabling interlaced video format (for V3D)


class TraceLogLevel(IntEnum):
	"""Trace log level"""
	LOG_ALL: int = 0  # Display all logs
	LOG_TRACE: int = 1  # Trace logging, intended for internal use only
	LOG_DEBUG: int = 2  # Debug logging, used for internal debugging, it should be disabled on release builds
	LOG_INFO: int = 3  # Info logging, used for program execution info
	LOG_WARNING: int = 4  # Warning logging, used on recoverable failures
	LOG_ERROR: int = 5  # Error logging, used on unrecoverable failures
	LOG_FATAL: int = 6  # Fatal logging, used to abort program: exit(EXIT_FAILURE)
	LOG_NONE: int = 7  # Disable logging


class KeyboardKey(IntEnum):
	"""Keyboard keys (US keyboard layout)"""
	KEY_NULL: int = 0  # Key: NULL, used for no key pressed
	KEY_APOSTROPHE: int = 39  # Key: '
	KEY_COMMA: int = 44  # Key: ,
	KEY_MINUS: int = 45  # Key: -
	KEY_PERIOD: int = 46  # Key: .
	KEY_SLASH: int = 47  # Key: /
	KEY_ZERO: int = 48  # Key: 0
	KEY_ONE: int = 49  # Key: 1
	KEY_TWO: int = 50  # Key: 2
	KEY_THREE: int = 51  # Key: 3
	KEY_FOUR: int = 52  # Key: 4
	KEY_FIVE: int = 53  # Key: 5
	KEY_SIX: int = 54  # Key: 6
	KEY_SEVEN: int = 55  # Key: 7
	KEY_EIGHT: int = 56  # Key: 8
	KEY_NINE: int = 57  # Key: 9
	KEY_SEMICOLON: int = 59  # Key: ;
	KEY_EQUAL: int = 61  # Key: =
	KEY_A: int = 65  # Key: A | a
	KEY_B: int = 66  # Key: B | b
	KEY_C: int = 67  # Key: C | c
	KEY_D: int = 68  # Key: D | d
	KEY_E: int = 69  # Key: E | e
	KEY_F: int = 70  # Key: F | f
	KEY_G: int = 71  # Key: G | g
	KEY_H: int = 72  # Key: H | h
	KEY_I: int = 73  # Key: I | i
	KEY_J: int = 74  # Key: J | j
	KEY_K: int = 75  # Key: K | k
	KEY_L: int = 76  # Key: L | l
	KEY_M: int = 77  # Key: M | m
	KEY_N: int = 78  # Key: N | n
	KEY_O: int = 79  # Key: O | o
	KEY_P: int = 80  # Key: P | p
	KEY_Q: int = 81  # Key: Q | q
	KEY_R: int = 82  # Key: R | r
	KEY_S: int = 83  # Key: S | s
	KEY_T: int = 84  # Key: T | t
	KEY_U: int = 85  # Key: U | u
	KEY_V: int = 86  # Key: V | v
	KEY_W: int = 87  # Key: W | w
	KEY_X: int = 88  # Key: X | x
	KEY_Y: int = 89  # Key: Y | y
	KEY_Z: int = 90  # Key: Z | z
	KEY_LEFT_BRACKET: int = 91  # Key: [
	KEY_BACKSLASH: int = 92  # Key: '\'
	KEY_RIGHT_BRACKET: int = 93  # Key: ]
	KEY_GRAVE: int = 96  # Key: `
	KEY_SPACE: int = 32  # Key: Space
	KEY_ESCAPE: int = 256  # Key: Esc
	KEY_ENTER: int = 257  # Key: Enter
	KEY_TAB: int = 258  # Key: Tab
	KEY_BACKSPACE: int = 259  # Key: Backspace
	KEY_INSERT: int = 260  # Key: Ins
	KEY_DELETE: int = 261  # Key: Del
	KEY_RIGHT: int = 262  # Key: Cursor right
	KEY_LEFT: int = 263  # Key: Cursor left
	KEY_DOWN: int = 264  # Key: Cursor down
	KEY_UP: int = 265  # Key: Cursor up
	KEY_PAGE_UP: int = 266  # Key: Page up
	KEY_PAGE_DOWN: int = 267  # Key: Page down
	KEY_HOME: int = 268  # Key: Home
	KEY_END: int = 269  # Key: End
	KEY_CAPS_LOCK: int = 280  # Key: Caps lock
	KEY_SCROLL_LOCK: int = 281  # Key: Scroll down
	KEY_NUM_LOCK: int = 282  # Key: Num lock
	KEY_PRINT_SCREEN: int = 283  # Key: Print screen
	KEY_PAUSE: int = 284  # Key: Pause
	KEY_F1: int = 290  # Key: F1
	KEY_F2: int = 291  # Key: F2
	KEY_F3: int = 292  # Key: F3
	KEY_F4: int = 293  # Key: F4
	KEY_F5: int = 294  # Key: F5
	KEY_F6: int = 295  # Key: F6
	KEY_F7: int = 296  # Key: F7
	KEY_F8: int = 297  # Key: F8
	KEY_F9: int = 298  # Key: F9
	KEY_F10: int = 299  # Key: F10
	KEY_F11: int = 300  # Key: F11
	KEY_F12: int = 301  # Key: F12
	KEY_LEFT_SHIFT: int = 340  # Key: Shift left
	KEY_LEFT_CONTROL: int = 341  # Key: Control left
	KEY_LEFT_ALT: int = 342  # Key: Alt left
	KEY_LEFT_SUPER: int = 343  # Key: Super left
	KEY_RIGHT_SHIFT: int = 344  # Key: Shift right
	KEY_RIGHT_CONTROL: int = 345  # Key: Control right
	KEY_RIGHT_ALT: int = 346  # Key: Alt right
	KEY_RIGHT_SUPER: int = 347  # Key: Super right
	KEY_KB_MENU: int = 348  # Key: KB menu
	KEY_KP_0: int = 320  # Key: Keypad 0
	KEY_KP_1: int = 321  # Key: Keypad 1
	KEY_KP_2: int = 322  # Key: Keypad 2
	KEY_KP_3: int = 323  # Key: Keypad 3
	KEY_KP_4: int = 324  # Key: Keypad 4
	KEY_KP_5: int = 325  # Key: Keypad 5
	KEY_KP_6: int = 326  # Key: Keypad 6
	KEY_KP_7: int = 327  # Key: Keypad 7
	KEY_KP_8: int = 328  # Key: Keypad 8
	KEY_KP_9: int = 329  # Key: Keypad 9
	KEY_KP_DECIMAL: int = 330  # Key: Keypad .
	KEY_KP_DIVIDE: int = 331  # Key: Keypad /
	KEY_KP_MULTIPLY: int = 332  # Key: Keypad *
	KEY_KP_SUBTRACT: int = 333  # Key: Keypad -
	KEY_KP_ADD: int = 334  # Key: Keypad +
	KEY_KP_ENTER: int = 335  # Key: Keypad Enter
	KEY_KP_EQUAL: int = 336  # Key: Keypad =
	KEY_BACK: int = 4  # Key: Android back button
	KEY_MENU: int = 82  # Key: Android menu button
	KEY_VOLUME_UP: int = 24  # Key: Android volume up button
	KEY_VOLUME_DOWN: int = 25  # Key: Android volume down button


class MouseButton(IntEnum):
	"""Mouse buttons"""
	MOUSE_BUTTON_LEFT: int = 0  # Mouse button left
	MOUSE_BUTTON_RIGHT: int = 1  # Mouse button right
	MOUSE_BUTTON_MIDDLE: int = 2  # Mouse button middle (pressed wheel)
	MOUSE_BUTTON_SIDE: int = 3  # Mouse button side (advanced mouse device)
	MOUSE_BUTTON_EXTRA: int = 4  # Mouse button extra (advanced mouse device)
	MOUSE_BUTTON_FORWARD: int = 5  # Mouse button fordward (advanced mouse device)
	MOUSE_BUTTON_BACK: int = 6  # Mouse button back (advanced mouse device)


class MouseCursor(IntEnum):
	"""Mouse cursor"""
	MOUSE_CURSOR_DEFAULT: int = 0  # Default pointer shape
	MOUSE_CURSOR_ARROW: int = 1  # Arrow shape
	MOUSE_CURSOR_IBEAM: int = 2  # Text writing cursor shape
	MOUSE_CURSOR_CROSSHAIR: int = 3  # Cross shape
	MOUSE_CURSOR_POINTING_HAND: int = 4  # Pointing hand cursor
	MOUSE_CURSOR_RESIZE_EW: int = 5  # Horizontal resize/move arrow shape
	MOUSE_CURSOR_RESIZE_NS: int = 6  # Vertical resize/move arrow shape
	MOUSE_CURSOR_RESIZE_NWSE: int = 7  # Top-left to bottom-right diagonal resize/move arrow shape
	MOUSE_CURSOR_RESIZE_NESW: int = 8  # The top-right to bottom-left diagonal resize/move arrow shape
	MOUSE_CURSOR_RESIZE_ALL: int = 9  # The omni-directional resize/move cursor shape
	MOUSE_CURSOR_NOT_ALLOWED: int = 10  # The operation-not-allowed shape


class GamepadButton(IntEnum):
	"""Gamepad buttons"""
	GAMEPAD_BUTTON_UNKNOWN: int = 0  # Unknown button, just for error checking
	GAMEPAD_BUTTON_LEFT_FACE_UP: int = 1  # Gamepad left DPAD up button
	GAMEPAD_BUTTON_LEFT_FACE_RIGHT: int = 2  # Gamepad left DPAD right button
	GAMEPAD_BUTTON_LEFT_FACE_DOWN: int = 3  # Gamepad left DPAD down button
	GAMEPAD_BUTTON_LEFT_FACE_LEFT: int = 4  # Gamepad left DPAD left button
	GAMEPAD_BUTTON_RIGHT_FACE_UP: int = 5  # Gamepad right button up (i.e. PS3: Triangle, Xbox: Y)
	GAMEPAD_BUTTON_RIGHT_FACE_RIGHT: int = 6  # Gamepad right button right (i.e. PS3: Square, Xbox: X)
	GAMEPAD_BUTTON_RIGHT_FACE_DOWN: int = 7  # Gamepad right button down (i.e. PS3: Cross, Xbox: A)
	GAMEPAD_BUTTON_RIGHT_FACE_LEFT: int = 8  # Gamepad right button left (i.e. PS3: Circle, Xbox: B)
	GAMEPAD_BUTTON_LEFT_TRIGGER_1: int = 9  # Gamepad top/back trigger left (first), it could be a trailing button
	GAMEPAD_BUTTON_LEFT_TRIGGER_2: int = 10  # Gamepad top/back trigger left (second), it could be a trailing button
	GAMEPAD_BUTTON_RIGHT_TRIGGER_1: int = 11  # Gamepad top/back trigger right (one), it could be a trailing button
	GAMEPAD_BUTTON_RIGHT_TRIGGER_2: int = 12  # Gamepad top/back trigger right (second), it could be a trailing button
	GAMEPAD_BUTTON_MIDDLE_LEFT: int = 13  # Gamepad center buttons, left one (i.e. PS3: Select)
	GAMEPAD_BUTTON_MIDDLE: int = 14  # Gamepad center buttons, middle one (i.e. PS3: PS, Xbox: XBOX)
	GAMEPAD_BUTTON_MIDDLE_RIGHT: int = 15  # Gamepad center buttons, right one (i.e. PS3: Start)
	GAMEPAD_BUTTON_LEFT_THUMB: int = 16  # Gamepad joystick pressed button left
	GAMEPAD_BUTTON_RIGHT_THUMB: int = 17  # Gamepad joystick pressed button right


class GamepadAxis(IntEnum):
	"""Gamepad axis"""
	GAMEPAD_AXIS_LEFT_X: int = 0  # Gamepad left stick X axis
	GAMEPAD_AXIS_LEFT_Y: int = 1  # Gamepad left stick Y axis
	GAMEPAD_AXIS_RIGHT_X: int = 2  # Gamepad right stick X axis
	GAMEPAD_AXIS_RIGHT_Y: int = 3  # Gamepad right stick Y axis
	GAMEPAD_AXIS_LEFT_TRIGGER: int = 4  # Gamepad back trigger left, pressure level: [1..-1]
	GAMEPAD_AXIS_RIGHT_TRIGGER: int = 5  # Gamepad back trigger right, pressure level: [1..-1]


class MaterialMapIndex(IntEnum):
	"""Material map index"""
	MATERIAL_MAP_ALBEDO: int = 0  # Albedo material (same as: MATERIAL_MAP_DIFFUSE)
	MATERIAL_MAP_METALNESS: int = 1  # Metalness material (same as: MATERIAL_MAP_SPECULAR)
	MATERIAL_MAP_NORMAL: int = 2  # Normal material
	MATERIAL_MAP_ROUGHNESS: int = 3  # Roughness material
	MATERIAL_MAP_OCCLUSION: int = 4  # Ambient occlusion material
	MATERIAL_MAP_EMISSION: int = 5  # Emission material
	MATERIAL_MAP_HEIGHT: int = 6  # Heightmap material
	MATERIAL_MAP_CUBEMAP: int = 7  # Cubemap material (NOTE: Uses GL_TEXTURE_CUBE_MAP)
	MATERIAL_MAP_IRRADIANCE: int = 8  # Irradiance material (NOTE: Uses GL_TEXTURE_CUBE_MAP)
	MATERIAL_MAP_PREFILTER: int = 9  # Prefilter material (NOTE: Uses GL_TEXTURE_CUBE_MAP)
	MATERIAL_MAP_BRDF: int = 10  # Brdf material


class ShaderLocationIndex(IntEnum):
	"""Shader location index"""
	SHADER_LOC_VERTEX_POSITION: int = 0  # Shader location: vertex attribute: position
	SHADER_LOC_VERTEX_TEXCOORD01: int = 1  # Shader location: vertex attribute: texcoord01
	SHADER_LOC_VERTEX_TEXCOORD02: int = 2  # Shader location: vertex attribute: texcoord02
	SHADER_LOC_VERTEX_NORMAL: int = 3  # Shader location: vertex attribute: normal
	SHADER_LOC_VERTEX_TANGENT: int = 4  # Shader location: vertex attribute: tangent
	SHADER_LOC_VERTEX_COLOR: int = 5  # Shader location: vertex attribute: color
	SHADER_LOC_MATRIX_MVP: int = 6  # Shader location: matrix uniform: model-view-projection
	SHADER_LOC_MATRIX_VIEW: int = 7  # Shader location: matrix uniform: view (camera transform)
	SHADER_LOC_MATRIX_PROJECTION: int = 8  # Shader location: matrix uniform: projection
	SHADER_LOC_MATRIX_MODEL: int = 9  # Shader location: matrix uniform: model (transform)
	SHADER_LOC_MATRIX_NORMAL: int = 10  # Shader location: matrix uniform: normal
	SHADER_LOC_VECTOR_VIEW: int = 11  # Shader location: vector uniform: view
	SHADER_LOC_COLOR_DIFFUSE: int = 12  # Shader location: vector uniform: diffuse color
	SHADER_LOC_COLOR_SPECULAR: int = 13  # Shader location: vector uniform: specular color
	SHADER_LOC_COLOR_AMBIENT: int = 14  # Shader location: vector uniform: ambient color
	SHADER_LOC_MAP_ALBEDO: int = 15  # Shader location: sampler2d texture: albedo (same as: SHADER_LOC_MAP_DIFFUSE)
	SHADER_LOC_MAP_METALNESS: int = 16  # Shader location: sampler2d texture: metalness (same as: SHADER_LOC_MAP_SPECULAR)
	SHADER_LOC_MAP_NORMAL: int = 17  # Shader location: sampler2d texture: normal
	SHADER_LOC_MAP_ROUGHNESS: int = 18  # Shader location: sampler2d texture: roughness
	SHADER_LOC_MAP_OCCLUSION: int = 19  # Shader location: sampler2d texture: occlusion
	SHADER_LOC_MAP_EMISSION: int = 20  # Shader location: sampler2d texture: emission
	SHADER_LOC_MAP_HEIGHT: int = 21  # Shader location: sampler2d texture: height
	SHADER_LOC_MAP_CUBEMAP: int = 22  # Shader location: samplerCube texture: cubemap
	SHADER_LOC_MAP_IRRADIANCE: int = 23  # Shader location: samplerCube texture: irradiance
	SHADER_LOC_MAP_PREFILTER: int = 24  # Shader location: samplerCube texture: prefilter
	SHADER_LOC_MAP_BRDF: int = 25  # Shader location: sampler2d texture: brdf


class ShaderUniformDataType(IntEnum):
	"""Shader uniform data type"""
	SHADER_UNIFORM_FLOAT: int = 0  # Shader uniform type: float
	SHADER_UNIFORM_VEC2: int = 1  # Shader uniform type: vec2 (2 float)
	SHADER_UNIFORM_VEC3: int = 2  # Shader uniform type: vec3 (3 float)
	SHADER_UNIFORM_VEC4: int = 3  # Shader uniform type: vec4 (4 float)
	SHADER_UNIFORM_INT: int = 4  # Shader uniform type: int
	SHADER_UNIFORM_IVEC2: int = 5  # Shader uniform type: ivec2 (2 int)
	SHADER_UNIFORM_IVEC3: int = 6  # Shader uniform type: ivec3 (3 int)
	SHADER_UNIFORM_IVEC4: int = 7  # Shader uniform type: ivec4 (4 int)
	SHADER_UNIFORM_SAMPLER2D: int = 8  # Shader uniform type: sampler2d


class ShaderAttributeDataType(IntEnum):
	"""Shader attribute data types"""
	SHADER_ATTRIB_FLOAT: int = 0  # Shader attribute type: float
	SHADER_ATTRIB_VEC2: int = 1  # Shader attribute type: vec2 (2 float)
	SHADER_ATTRIB_VEC3: int = 2  # Shader attribute type: vec3 (3 float)
	SHADER_ATTRIB_VEC4: int = 3  # Shader attribute type: vec4 (4 float)


class PixelFormat(IntEnum):
	"""Pixel formats"""
	PIXELFORMAT_UNCOMPRESSED_GRAYSCALE: int = 1  # 8 bit per pixel (no alpha)
	PIXELFORMAT_UNCOMPRESSED_GRAY_ALPHA: int = 2  # 8*2 bpp (2 channels)
	PIXELFORMAT_UNCOMPRESSED_R5G6B5: int = 3  # 16 bpp
	PIXELFORMAT_UNCOMPRESSED_R8G8B8: int = 4  # 24 bpp
	PIXELFORMAT_UNCOMPRESSED_R5G5B5A1: int = 5  # 16 bpp (1 bit alpha)
	PIXELFORMAT_UNCOMPRESSED_R4G4B4A4: int = 6  # 16 bpp (4 bit alpha)
	PIXELFORMAT_UNCOMPRESSED_R8G8B8A8: int = 7  # 32 bpp
	PIXELFORMAT_UNCOMPRESSED_R32: int = 8  # 32 bpp (1 channel - float)
	PIXELFORMAT_UNCOMPRESSED_R32G32B32: int = 9  # 32*3 bpp (3 channels - float)
	PIXELFORMAT_UNCOMPRESSED_R32G32B32A32: int = 10  # 32*4 bpp (4 channels - float)
	PIXELFORMAT_COMPRESSED_DXT1_RGB: int = 11  # 4 bpp (no alpha)
	PIXELFORMAT_COMPRESSED_DXT1_RGBA: int = 12  # 4 bpp (1 bit alpha)
	PIXELFORMAT_COMPRESSED_DXT3_RGBA: int = 13  # 8 bpp
	PIXELFORMAT_COMPRESSED_DXT5_RGBA: int = 14  # 8 bpp
	PIXELFORMAT_COMPRESSED_ETC1_RGB: int = 15  # 4 bpp
	PIXELFORMAT_COMPRESSED_ETC2_RGB: int = 16  # 4 bpp
	PIXELFORMAT_COMPRESSED_ETC2_EAC_RGBA: int = 17  # 8 bpp
	PIXELFORMAT_COMPRESSED_PVRT_RGB: int = 18  # 4 bpp
	PIXELFORMAT_COMPRESSED_PVRT_RGBA: int = 19  # 4 bpp
	PIXELFORMAT_COMPRESSED_ASTC_4x4_RGBA: int = 20  # 8 bpp
	PIXELFORMAT_COMPRESSED_ASTC_8x8_RGBA: int = 21  # 2 bpp


class TextureFilter(IntEnum):
	"""Texture parameters: filter mode"""
	TEXTURE_FILTER_POINT: int = 0  # No filter, just pixel approximation
	TEXTURE_FILTER_BILINEAR: int = 1  # Linear filtering
	TEXTURE_FILTER_TRILINEAR: int = 2  # Trilinear filtering (linear with mipmaps)
	TEXTURE_FILTER_ANISOTROPIC_4X: int = 3  # Anisotropic filtering 4x
	TEXTURE_FILTER_ANISOTROPIC_8X: int = 4  # Anisotropic filtering 8x
	TEXTURE_FILTER_ANISOTROPIC_16X: int = 5  # Anisotropic filtering 16x


class TextureWrap(IntEnum):
	"""Texture parameters: wrap mode"""
	TEXTURE_WRAP_REPEAT: int = 0  # Repeats texture in tiled mode
	TEXTURE_WRAP_CLAMP: int = 1  # Clamps texture to edge pixel in tiled mode
	TEXTURE_WRAP_MIRROR_REPEAT: int = 2  # Mirrors and repeats the texture in tiled mode
	TEXTURE_WRAP_MIRROR_CLAMP: int = 3  # Mirrors and clamps to border the texture in tiled mode


class CubemapLayout(IntEnum):
	"""Cubemap layouts"""
	CUBEMAP_LAYOUT_AUTO_DETECT: int = 0  # Automatically detect layout type
	CUBEMAP_LAYOUT_LINE_VERTICAL: int = 1  # Layout is defined by a vertical line with faces
	CUBEMAP_LAYOUT_LINE_HORIZONTAL: int = 2  # Layout is defined by an horizontal line with faces
	CUBEMAP_LAYOUT_CROSS_THREE_BY_FOUR: int = 3  # Layout is defined by a 3x4 cross with cubemap faces
	CUBEMAP_LAYOUT_CROSS_FOUR_BY_THREE: int = 4  # Layout is defined by a 4x3 cross with cubemap faces
	CUBEMAP_LAYOUT_PANORAMA: int = 5  # Layout is defined by a panorama image (equirectangular map)


class FontType(IntEnum):
	"""Font type, defines generation method"""
	FONT_DEFAULT: int = 0  # Default font generation, anti-aliased
	FONT_BITMAP: int = 1  # Bitmap font generation, no anti-aliasing
	FONT_SDF: int = 2  # SDF font generation, requires external shader


class BlendMode(IntEnum):
	"""Color blending modes (pre-defined)"""
	BLEND_ALPHA: int = 0  # Blend textures considering alpha (default)
	BLEND_ADDITIVE: int = 1  # Blend textures adding colors
	BLEND_MULTIPLIED: int = 2  # Blend textures multiplying colors
	BLEND_ADD_COLORS: int = 3  # Blend textures adding colors (alternative)
	BLEND_SUBTRACT_COLORS: int = 4  # Blend textures subtracting colors (alternative)
	BLEND_ALPHA_PREMULTIPLY: int = 5  # Blend premultiplied textures considering alpha
	BLEND_CUSTOM: int = 6  # Blend textures using custom src/dst factors (use rlSetBlendMode())


class Gesture(IntEnum):
	"""Gesture"""
	GESTURE_NONE: int = 0  # No gesture
	GESTURE_TAP: int = 1  # Tap gesture
	GESTURE_DOUBLETAP: int = 2  # Double tap gesture
	GESTURE_HOLD: int = 4  # Hold gesture
	GESTURE_DRAG: int = 8  # Drag gesture
	GESTURE_SWIPE_RIGHT: int = 16  # Swipe right gesture
	GESTURE_SWIPE_LEFT: int = 32  # Swipe left gesture
	GESTURE_SWIPE_UP: int = 64  # Swipe up gesture
	GESTURE_SWIPE_DOWN: int = 128  # Swipe down gesture
	GESTURE_PINCH_IN: int = 256  # Pinch in gesture
	GESTURE_PINCH_OUT: int = 512  # Pinch out gesture


class CameraMode(IntEnum):
	"""Camera system modes"""
	CAMERA_CUSTOM: int = 0  # Custom camera
	CAMERA_FREE: int = 1  # Free camera
	CAMERA_ORBITAL: int = 2  # Orbital camera
	CAMERA_FIRST_PERSON: int = 3  # First person camera
	CAMERA_THIRD_PERSON: int = 4  # Third person camera


class CameraProjection(IntEnum):
	"""Camera projection"""
	CAMERA_PERSPECTIVE: int = 0  # Perspective projection
	CAMERA_ORTHOGRAPHIC: int = 1  # Orthographic projection


class NPatchLayout(IntEnum):
	"""N-patch layout"""
	NPATCH_NINE_PATCH: int = 0  # Npatch layout: 3x3 tiles
	NPATCH_THREE_PATCH_VERTICAL: int = 1  # Npatch layout: 1x3 tiles
	NPATCH_THREE_PATCH_HORIZONTAL: int = 2  # Npatch layout: 3x1 tiles


class GuiState(IntEnum):
	"""Gui control state"""
	STATE_NORMAL: int = 0  # 
	STATE_FOCUSED: int = 1  # 
	STATE_PRESSED: int = 2  # 
	STATE_DISABLED: int = 3  # 


class GuiTextAlignment(IntEnum):
	"""Gui control text alignment"""
	TEXT_ALIGN_LEFT: int = 0  # 
	TEXT_ALIGN_CENTER: int = 1  # 
	TEXT_ALIGN_RIGHT: int = 2  # 


class GuiControl(IntEnum):
	"""Gui controls"""
	DEFAULT: int = 0  # 
	LABEL: int = 1  # Used also for: LABELBUTTON
	BUTTON: int = 2  # 
	TOGGLE: int = 3  # Used also for: TOGGLEGROUP
	SLIDER: int = 4  # Used also for: SLIDERBAR
	PROGRESSBAR: int = 5  # 
	CHECKBOX: int = 6  # 
	COMBOBOX: int = 7  # 
	DROPDOWNBOX: int = 8  # 
	TEXTBOX: int = 9  # Used also for: TEXTBOXMULTI
	VALUEBOX: int = 10  # 
	SPINNER: int = 11  # Uses: BUTTON, VALUEBOX
	LISTVIEW: int = 12  # 
	COLORPICKER: int = 13  # 
	SCROLLBAR: int = 14  # 
	STATUSBAR: int = 15  # 


class GuiControlProperty(IntEnum):
	"""Gui base properties for every control"""
	BORDER_COLOR_NORMAL: int = 0  # 
	BASE_COLOR_NORMAL: int = 1  # 
	TEXT_COLOR_NORMAL: int = 2  # 
	BORDER_COLOR_FOCUSED: int = 3  # 
	BASE_COLOR_FOCUSED: int = 4  # 
	TEXT_COLOR_FOCUSED: int = 5  # 
	BORDER_COLOR_PRESSED: int = 6  # 
	BASE_COLOR_PRESSED: int = 7  # 
	TEXT_COLOR_PRESSED: int = 8  # 
	BORDER_COLOR_DISABLED: int = 9  # 
	BASE_COLOR_DISABLED: int = 10  # 
	TEXT_COLOR_DISABLED: int = 11  # 
	BORDER_WIDTH: int = 12  # 
	TEXT_PADDING: int = 13  # 
	TEXT_ALIGNMENT: int = 14  # 
	RESERVED: int = 15  # 


class GuiDefaultProperty(IntEnum):
	"""DEFAULT extended properties"""
	TEXT_SIZE: int = 16  # Text size (glyphs max height)
	TEXT_SPACING: int = 17  # Text spacing between glyphs
	LINE_COLOR: int = 18  # Line control color
	BACKGROUND_COLOR: int = 19  # Background color


class GuiToggleProperty(IntEnum):
	"""Toggle/ToggleGroup"""
	GROUP_PADDING: int = 16  # ToggleGroup separation between toggles


class GuiSliderProperty(IntEnum):
	"""Slider/SliderBar"""
	SLIDER_WIDTH: int = 16  # Slider size of internal bar
	SLIDER_PADDING: int = 17  # Slider/SliderBar internal bar padding


class GuiProgressBarProperty(IntEnum):
	"""ProgressBar"""
	PROGRESS_PADDING: int = 16  # ProgressBar internal padding


class GuiScrollBarProperty(IntEnum):
	"""ScrollBar"""
	ARROWS_SIZE: int = 16  # 
	ARROWS_VISIBLE: int = 17  # 
	SCROLL_SLIDER_PADDING: int = 18  # (SLIDERBAR, SLIDER_PADDING)
	SCROLL_SLIDER_SIZE: int = 19  # 
	SCROLL_PADDING: int = 20  # 
	SCROLL_SPEED: int = 21  # 


class GuiCheckBoxProperty(IntEnum):
	"""CheckBox"""
	CHECK_PADDING: int = 16  # CheckBox internal check padding


class GuiComboBoxProperty(IntEnum):
	"""ComboBox"""
	COMBO_BUTTON_WIDTH: int = 16  # ComboBox right button width
	COMBO_BUTTON_SPACING: int = 17  # ComboBox button separation


class GuiDropdownBoxProperty(IntEnum):
	"""DropdownBox"""
	ARROW_PADDING: int = 16  # DropdownBox arrow separation from border and items
	DROPDOWN_ITEMS_SPACING: int = 17  # DropdownBox items separation


class GuiTextBoxProperty(IntEnum):
	"""TextBox/TextBoxMulti/ValueBox/Spinner"""
	TEXT_INNER_PADDING: int = 16  # TextBox/TextBoxMulti/ValueBox/Spinner inner text padding
	TEXT_LINES_SPACING: int = 17  # TextBoxMulti lines separation


class GuiSpinnerProperty(IntEnum):
	"""Spinner"""
	SPIN_BUTTON_WIDTH: int = 16  # Spinner left/right buttons width
	SPIN_BUTTON_SPACING: int = 17  # Spinner buttons separation


class GuiListViewProperty(IntEnum):
	"""ListView"""
	LIST_ITEMS_HEIGHT: int = 16  # ListView items height
	LIST_ITEMS_SPACING: int = 17  # ListView items separation
	SCROLLBAR_WIDTH: int = 18  # ListView scrollbar size (usually width)
	SCROLLBAR_SIDE: int = 19  # ListView scrollbar side (0-left, 1-right)


class GuiColorPickerProperty(IntEnum):
	"""ColorPicker"""
	COLOR_SELECTOR_SIZE: int = 16  # 
	HUEBAR_WIDTH: int = 17  # ColorPicker right hue bar width
	HUEBAR_PADDING: int = 18  # ColorPicker right hue bar separation from panel
	HUEBAR_SELECTOR_HEIGHT: int = 19  # ColorPicker right hue bar selector height
	HUEBAR_SELECTOR_OVERFLOW: int = 20  # ColorPicker right hue bar selector overflow


class GuiIconName(IntEnum):
	""""""
	ICON_NONE: int = 0  # 
	ICON_FOLDER_FILE_OPEN: int = 1  # 
	ICON_FILE_SAVE_CLASSIC: int = 2  # 
	ICON_FOLDER_OPEN: int = 3  # 
	ICON_FOLDER_SAVE: int = 4  # 
	ICON_FILE_OPEN: int = 5  # 
	ICON_FILE_SAVE: int = 6  # 
	ICON_FILE_EXPORT: int = 7  # 
	ICON_FILE_ADD: int = 8  # 
	ICON_FILE_DELETE: int = 9  # 
	ICON_FILETYPE_TEXT: int = 10  # 
	ICON_FILETYPE_AUDIO: int = 11  # 
	ICON_FILETYPE_IMAGE: int = 12  # 
	ICON_FILETYPE_PLAY: int = 13  # 
	ICON_FILETYPE_VIDEO: int = 14  # 
	ICON_FILETYPE_INFO: int = 15  # 
	ICON_FILE_COPY: int = 16  # 
	ICON_FILE_CUT: int = 17  # 
	ICON_FILE_PASTE: int = 18  # 
	ICON_CURSOR_HAND: int = 19  # 
	ICON_CURSOR_POINTER: int = 20  # 
	ICON_CURSOR_CLASSIC: int = 21  # 
	ICON_PENCIL: int = 22  # 
	ICON_PENCIL_BIG: int = 23  # 
	ICON_BRUSH_CLASSIC: int = 24  # 
	ICON_BRUSH_PAINTER: int = 25  # 
	ICON_WATER_DROP: int = 26  # 
	ICON_COLOR_PICKER: int = 27  # 
	ICON_RUBBER: int = 28  # 
	ICON_COLOR_BUCKET: int = 29  # 
	ICON_TEXT_T: int = 30  # 
	ICON_TEXT_A: int = 31  # 
	ICON_SCALE: int = 32  # 
	ICON_RESIZE: int = 33  # 
	ICON_FILTER_POINT: int = 34  # 
	ICON_FILTER_BILINEAR: int = 35  # 
	ICON_CROP: int = 36  # 
	ICON_CROP_ALPHA: int = 37  # 
	ICON_SQUARE_TOGGLE: int = 38  # 
	ICON_SYMMETRY: int = 39  # 
	ICON_SYMMETRY_HORIZONTAL: int = 40  # 
	ICON_SYMMETRY_VERTICAL: int = 41  # 
	ICON_LENS: int = 42  # 
	ICON_LENS_BIG: int = 43  # 
	ICON_EYE_ON: int = 44  # 
	ICON_EYE_OFF: int = 45  # 
	ICON_FILTER_TOP: int = 46  # 
	ICON_FILTER: int = 47  # 
	ICON_TARGET_POINT: int = 48  # 
	ICON_TARGET_SMALL: int = 49  # 
	ICON_TARGET_BIG: int = 50  # 
	ICON_TARGET_MOVE: int = 51  # 
	ICON_CURSOR_MOVE: int = 52  # 
	ICON_CURSOR_SCALE: int = 53  # 
	ICON_CURSOR_SCALE_RIGHT: int = 54  # 
	ICON_CURSOR_SCALE_LEFT: int = 55  # 
	ICON_UNDO: int = 56  # 
	ICON_REDO: int = 57  # 
	ICON_REREDO: int = 58  # 
	ICON_MUTATE: int = 59  # 
	ICON_ROTATE: int = 60  # 
	ICON_REPEAT: int = 61  # 
	ICON_SHUFFLE: int = 62  # 
	ICON_EMPTYBOX: int = 63  # 
	ICON_TARGET: int = 64  # 
	ICON_TARGET_SMALL_FILL: int = 65  # 
	ICON_TARGET_BIG_FILL: int = 66  # 
	ICON_TARGET_MOVE_FILL: int = 67  # 
	ICON_CURSOR_MOVE_FILL: int = 68  # 
	ICON_CURSOR_SCALE_FILL: int = 69  # 
	ICON_CURSOR_SCALE_RIGHT_FILL: int = 70  # 
	ICON_CURSOR_SCALE_LEFT_FILL: int = 71  # 
	ICON_UNDO_FILL: int = 72  # 
	ICON_REDO_FILL: int = 73  # 
	ICON_REREDO_FILL: int = 74  # 
	ICON_MUTATE_FILL: int = 75  # 
	ICON_ROTATE_FILL: int = 76  # 
	ICON_REPEAT_FILL: int = 77  # 
	ICON_SHUFFLE_FILL: int = 78  # 
	ICON_EMPTYBOX_SMALL: int = 79  # 
	ICON_BOX: int = 80  # 
	ICON_BOX_TOP: int = 81  # 
	ICON_BOX_TOP_RIGHT: int = 82  # 
	ICON_BOX_RIGHT: int = 83  # 
	ICON_BOX_BOTTOM_RIGHT: int = 84  # 
	ICON_BOX_BOTTOM: int = 85  # 
	ICON_BOX_BOTTOM_LEFT: int = 86  # 
	ICON_BOX_LEFT: int = 87  # 
	ICON_BOX_TOP_LEFT: int = 88  # 
	ICON_BOX_CENTER: int = 89  # 
	ICON_BOX_CIRCLE_MASK: int = 90  # 
	ICON_POT: int = 91  # 
	ICON_ALPHA_MULTIPLY: int = 92  # 
	ICON_ALPHA_CLEAR: int = 93  # 
	ICON_DITHERING: int = 94  # 
	ICON_MIPMAPS: int = 95  # 
	ICON_BOX_GRID: int = 96  # 
	ICON_GRID: int = 97  # 
	ICON_BOX_CORNERS_SMALL: int = 98  # 
	ICON_BOX_CORNERS_BIG: int = 99  # 
	ICON_FOUR_BOXES: int = 100  # 
	ICON_GRID_FILL: int = 101  # 
	ICON_BOX_MULTISIZE: int = 102  # 
	ICON_ZOOM_SMALL: int = 103  # 
	ICON_ZOOM_MEDIUM: int = 104  # 
	ICON_ZOOM_BIG: int = 105  # 
	ICON_ZOOM_ALL: int = 106  # 
	ICON_ZOOM_CENTER: int = 107  # 
	ICON_BOX_DOTS_SMALL: int = 108  # 
	ICON_BOX_DOTS_BIG: int = 109  # 
	ICON_BOX_CONCENTRIC: int = 110  # 
	ICON_BOX_GRID_BIG: int = 111  # 
	ICON_OK_TICK: int = 112  # 
	ICON_CROSS: int = 113  # 
	ICON_ARROW_LEFT: int = 114  # 
	ICON_ARROW_RIGHT: int = 115  # 
	ICON_ARROW_DOWN: int = 116  # 
	ICON_ARROW_UP: int = 117  # 
	ICON_ARROW_LEFT_FILL: int = 118  # 
	ICON_ARROW_RIGHT_FILL: int = 119  # 
	ICON_ARROW_DOWN_FILL: int = 120  # 
	ICON_ARROW_UP_FILL: int = 121  # 
	ICON_AUDIO: int = 122  # 
	ICON_FX: int = 123  # 
	ICON_WAVE: int = 124  # 
	ICON_WAVE_SINUS: int = 125  # 
	ICON_WAVE_SQUARE: int = 126  # 
	ICON_WAVE_TRIANGULAR: int = 127  # 
	ICON_CROSS_SMALL: int = 128  # 
	ICON_PLAYER_PREVIOUS: int = 129  # 
	ICON_PLAYER_PLAY_BACK: int = 130  # 
	ICON_PLAYER_PLAY: int = 131  # 
	ICON_PLAYER_PAUSE: int = 132  # 
	ICON_PLAYER_STOP: int = 133  # 
	ICON_PLAYER_NEXT: int = 134  # 
	ICON_PLAYER_RECORD: int = 135  # 
	ICON_MAGNET: int = 136  # 
	ICON_LOCK_CLOSE: int = 137  # 
	ICON_LOCK_OPEN: int = 138  # 
	ICON_CLOCK: int = 139  # 
	ICON_TOOLS: int = 140  # 
	ICON_GEAR: int = 141  # 
	ICON_GEAR_BIG: int = 142  # 
	ICON_BIN: int = 143  # 
	ICON_HAND_POINTER: int = 144  # 
	ICON_LASER: int = 145  # 
	ICON_COIN: int = 146  # 
	ICON_EXPLOSION: int = 147  # 
	ICON_1UP: int = 148  # 
	ICON_PLAYER: int = 149  # 
	ICON_PLAYER_JUMP: int = 150  # 
	ICON_KEY: int = 151  # 
	ICON_DEMON: int = 152  # 
	ICON_TEXT_POPUP: int = 153  # 
	ICON_GEAR_EX: int = 154  # 
	ICON_CRACK: int = 155  # 
	ICON_CRACK_POINTS: int = 156  # 
	ICON_STAR: int = 157  # 
	ICON_DOOR: int = 158  # 
	ICON_EXIT: int = 159  # 
	ICON_MODE_2D: int = 160  # 
	ICON_MODE_3D: int = 161  # 
	ICON_CUBE: int = 162  # 
	ICON_CUBE_FACE_TOP: int = 163  # 
	ICON_CUBE_FACE_LEFT: int = 164  # 
	ICON_CUBE_FACE_FRONT: int = 165  # 
	ICON_CUBE_FACE_BOTTOM: int = 166  # 
	ICON_CUBE_FACE_RIGHT: int = 167  # 
	ICON_CUBE_FACE_BACK: int = 168  # 
	ICON_CAMERA: int = 169  # 
	ICON_SPECIAL: int = 170  # 
	ICON_LINK_NET: int = 171  # 
	ICON_LINK_BOXES: int = 172  # 
	ICON_LINK_MULTI: int = 173  # 
	ICON_LINK: int = 174  # 
	ICON_LINK_BROKE: int = 175  # 
	ICON_TEXT_NOTES: int = 176  # 
	ICON_NOTEBOOK: int = 177  # 
	ICON_SUITCASE: int = 178  # 
	ICON_SUITCASE_ZIP: int = 179  # 
	ICON_MAILBOX: int = 180  # 
	ICON_MONITOR: int = 181  # 
	ICON_PRINTER: int = 182  # 
	ICON_PHOTO_CAMERA: int = 183  # 
	ICON_PHOTO_CAMERA_FLASH: int = 184  # 
	ICON_HOUSE: int = 185  # 
	ICON_HEART: int = 186  # 
	ICON_CORNER: int = 187  # 
	ICON_VERTICAL_BARS: int = 188  # 
	ICON_VERTICAL_BARS_FILL: int = 189  # 
	ICON_LIFE_BARS: int = 190  # 
	ICON_INFO: int = 191  # 
	ICON_CROSSLINE: int = 192  # 
	ICON_HELP: int = 193  # 
	ICON_FILETYPE_ALPHA: int = 194  # 
	ICON_FILETYPE_HOME: int = 195  # 
	ICON_LAYERS_VISIBLE: int = 196  # 
	ICON_LAYERS: int = 197  # 
	ICON_WINDOW: int = 198  # 
	ICON_HIDPI: int = 199  # 
	ICON_FILETYPE_BINARY: int = 200  # 
	ICON_HEX: int = 201  # 
	ICON_SHIELD: int = 202  # 
	ICON_FILE_NEW: int = 203  # 
	ICON_FOLDER_ADD: int = 204  # 
	ICON_ALARM: int = 205  # 
	ICON_206: int = 206  # 
	ICON_207: int = 207  # 
	ICON_208: int = 208  # 
	ICON_209: int = 209  # 
	ICON_210: int = 210  # 
	ICON_211: int = 211  # 
	ICON_212: int = 212  # 
	ICON_213: int = 213  # 
	ICON_214: int = 214  # 
	ICON_215: int = 215  # 
	ICON_216: int = 216  # 
	ICON_217: int = 217  # 
	ICON_218: int = 218  # 
	ICON_219: int = 219  # 
	ICON_220: int = 220  # 
	ICON_221: int = 221  # 
	ICON_222: int = 222  # 
	ICON_223: int = 223  # 
	ICON_224: int = 224  # 
	ICON_225: int = 225  # 
	ICON_226: int = 226  # 
	ICON_227: int = 227  # 
	ICON_228: int = 228  # 
	ICON_229: int = 229  # 
	ICON_230: int = 230  # 
	ICON_231: int = 231  # 
	ICON_232: int = 232  # 
	ICON_233: int = 233  # 
	ICON_234: int = 234  # 
	ICON_235: int = 235  # 
	ICON_236: int = 236  # 
	ICON_237: int = 237  # 
	ICON_238: int = 238  # 
	ICON_239: int = 239  # 
	ICON_240: int = 240  # 
	ICON_241: int = 241  # 
	ICON_242: int = 242  # 
	ICON_243: int = 243  # 
	ICON_244: int = 244  # 
	ICON_245: int = 245  # 
	ICON_246: int = 246  # 
	ICON_247: int = 247  # 
	ICON_248: int = 248  # 
	ICON_249: int = 249  # 
	ICON_250: int = 250  # 
	ICON_251: int = 251  # 
	ICON_252: int = 252  # 
	ICON_253: int = 253  # 
	ICON_254: int = 254  # 
	ICON_255: int = 255  # 


